@@ -175,6 +175,8 @@ def get_btid(btid):
 
 _D('Wireless Keyboard EX110', codename='EX110', protocol=1.0, wpid='0055', registers=(_R.battery_status, ))
 _D('Wireless Keyboard S510', codename='S510', protocol=1.0, wpid='0056', registers=(_R.battery_status, ))
+_D('Wireless Wave Keyboard K550', codename='K550', protocol=1.0, wpid='0060', registers=(_R.battery_status, ),
+   settings=[_ST.RegisterFnSwap])
 _D('Wireless Keyboard EX100', codename='EX100', protocol=1.0, wpid='0065', registers=(_R.battery_status, ))
 _D('Wireless Keyboard MK300', codename='MK300', protocol=1.0, wpid='0068', registers=(_R.battery_status, ))
 _D('Number Pad N545', codename='N545', protocol=1.0, wpid='2006', registers=(_R.battery_status, ))
@@ -207,11 +209,13 @@ _D('G213 Prodigy Gaming Keyboard', codename='G213', usbid=0xc336, interface=1)
 _D('G512 RGB Mechanical Gaming Keyboard', codename='G512', usbid=0xc33c, interface=1)
 _D('G815 Mechanical Keyboard', codename='G815', usbid=0xc33f, interface=1)
 _D('diNovo Edge Keyboard', codename='diNovo', protocol=1.0, wpid='C714', settings=[_ST.RegisterFnSwap])
+_D('K845 Mechanical Keyboard', codename='K845', usbid=0xc341, interface=3)
 
 # Mice
 
 _D('LX5 Cordless Mouse', codename='LX5', protocol=1.0, wpid='0036', registers=(_R.battery_status, ))
 _D('LX7 Cordless Laser Mouse', codename='LX7', protocol=1.0, wpid='0039', registers=(_R.battery_status, ))
+_D('Wireless Wave Mouse M550', codename='M550', protocol=1.0, wpid='003C', registers=(_R.battery_status, ))
 _D('Wireless Mouse EX100', codename='EX100m', protocol=1.0, wpid='003F', registers=(_R.battery_status, ))
 _D('Wireless Mouse M30', codename='M30', protocol=1.0, wpid='0085', registers=(_R.battery_status, ))
 _D('MX610 Laser Cordless Mouse', codename='MX610', protocol=1.0, wpid='1001', registers=(_R.battery_status, ))
@@ -262,7 +266,7 @@ _D('G700s Gaming Mouse', codename='G700s', protocol=1.0, wpid='102A', usbid=0xc0
     registers=(_R.battery_status, _R.three_leds,), settings=[_ST.RegisterSmoothScroll, _ST.RegisterSideScroll])
 _D('Couch Mouse M515', codename='M515', protocol=2.0, wpid='4007')
 _D('Wireless Mouse M175', codename='M175', protocol=2.0, wpid='4008')
-_D('Wireless Mouse M325', codename='M325', protocol=2.0, wpid='400A', settings=[_ST.HiResScroll])
+_D('Wireless Mouse M325', codename='M325', protocol=2.0, wpid='400A')
 _D('Wireless Mouse M525', codename='M525', protocol=2.0, wpid='4013')
 _D('Wireless Mouse M345', codename='M345', protocol=2.0, wpid='4017')
 _D('Wireless Mouse M187', codename='M187', protocol=2.0, wpid='4019')
@@ -317,6 +321,7 @@ _D('Wireless Trackball M570', codename='M570')
 
 _D('Wireless Touchpad', codename='Wireless Touch', protocol=2.0, wpid='4011')
 _D('Wireless Rechargeable Touchpad T650', codename='T650', protocol=2.0, wpid='4101')
+_D('G Powerplay', codename='Powerplay', protocol=2.0, kind=_DK.touchpad, wpid='405F')  # To override self-identification
 
 # Headset
 
