@@ -18,17 +18,15 @@
 
 # Logitech Unifying Receiver API.
 
+import logging
 import threading as _threading
 
-from logging import DEBUG as _DEBUG
-from logging import ERROR as _ERROR
-from logging import INFO as _INFO
-from logging import WARNING as _WARNING
-from logging import getLogger
 from struct import pack as _pack
 from struct import unpack as _unpack
 from typing import List
 
+import yaml as _yaml
+
 from . import special_keys
 from .common import BATTERY_APPROX as _BATTERY_APPROX
 from .common import FirmwareInfo as _FirmwareInfo
@@ -37,10 +35,18 @@ from .common import NamedInt as _NamedInt
 from .common import NamedInts as _NamedInts
 from .common import UnsortedNamedInts as _UnsortedNamedInts
 from .common import bytes2int as _bytes2int
+from .common import crc16 as _crc16
 from .common import int2bytes as _int2bytes
+from .i18n import _
 
-_log = getLogger(__name__)
-del getLogger
+logger = logging.getLogger(__name__)
+
+
+def hexint_presenter(dumper, data):
+    return dumper.represent_int(hex(data))
+
+
+_yaml.add_representer(int, hexint_presenter)
 
 #
 #
@@ -79,6 +85,7 @@ FEATURE = _NamedInts(
     UNIFIED_BATTERY=0x1004,
     CHARGING_CONTROL=0x1010,
     LED_CONTROL=0x1300,
+    FORCE_PAIRING=0x1500,
     GENERIC_TEST=0x1800,
     DEVICE_RESET=0x1802,
     OOBSTATE=0x1805,
@@ -115,6 +122,7 @@ FEATURE = _NamedInts(
     THUMB_WHEEL=0x2150,
     MOUSE_POINTER=0x2200,
     ADJUSTABLE_DPI=0x2201,
+    EXTENDED_ADJUSTABLE_DPI=0x2202,
     POINTER_SPEED=0x2205,
     ANGLE_SNAPPING=0x2230,
     SURFACE_TUNING=0x2240,
@@ -154,6 +162,7 @@ FEATURE = _NamedInts(
     MR=0x8030,
     BRIGHTNESS_CONTROL=0x8040,
     REPORT_RATE=0x8060,
+    EXTENDED_ADJUSTABLE_REPORT_RATE=0x8061,
     COLOR_LED_EFFECTS=0x8070,
     RGB_EFFECTS=0x8071,
     PER_KEY_LIGHTING=0x8080,
@@ -257,7 +266,7 @@ class FeaturesArray(dict):
             if fs_index:
                 count = self.device.request(fs_index << 8)
                 if count is None:
-                    _log.warn('FEATURE_SET found, but failed to read features count')
+                    logger.warn('FEATURE_SET found, but failed to read features count')
                     return False
                 else:
                     self.count = ord(count[:1]) + 1  # ROOT feature not included in count
@@ -441,8 +450,8 @@ class ReprogrammableKeyV4(ReprogrammableKey):
             mapped_data = feature_request(self._device, FEATURE.REPROG_CONTROLS_V4, 0x20, *tuple(_pack('!H', self._cid)))
             if mapped_data:
                 cid, mapping_flags_1, mapped_to = _unpack('!HBH', mapped_data[:5])
-                if cid != self._cid and _log.isEnabledFor(_WARNING):
-                    _log.warn(
+                if cid != self._cid and logger.isEnabledFor(logging.WARNING):
+                    logger.warn(
                         f'REPROG_CONTROLS_V4 endpoint getCidReporting on device {self._device} replied ' +
                         f'with a different control ID ({cid}) than requested ({self._cid}).'
                     )
@@ -455,8 +464,8 @@ class ReprogrammableKeyV4(ReprogrammableKey):
             else:
                 raise FeatureCallError(msg='No reply from device.')
         except FeatureCallError:  # if the key hasn't ever been configured then the read may fail so only produce a warning
-            if _log.isEnabledFor(_WARNING):
-                _log.warn(
+            if logger.isEnabledFor(logging.WARNING):
+                logger.warn(
                     f'Feature Call Error in _getCidReporting on device {self._device} for cid {self._cid} - use defaults'
                 )
             # Clear flags and set mapping target to self
@@ -513,8 +522,8 @@ class ReprogrammableKeyV4(ReprogrammableKey):
         # TODO: to fully support version 4 of REPROG_CONTROLS_V4, append `(bfield >> 8) & 0xff` here.
         # But older devices might behave oddly given that byte, so we don't send it.
         ret = feature_request(self._device, FEATURE.REPROG_CONTROLS_V4, 0x30, *pkt)
-        if ret is None or _unpack('!BBBBB', ret[:5]) != pkt and _log.isEnabledFor(_DEBUG):
-            _log.debug(f"REPROG_CONTROLS_v4 setCidReporting on device {self._device} didn't echo request packet.")
+        if ret is None or _unpack('!BBBBB', ret[:5]) != pkt and logger.isEnabledFor(logging.DEBUG):
+            logger.debug(f"REPROG_CONTROLS_v4 setCidReporting on device {self._device} didn't echo request packet.")
 
 
 class PersistentRemappableAction():
@@ -594,8 +603,8 @@ class KeysArray:
         elif FEATURE.REPROG_CONTROLS_V2 in self.device.features:
             self.keyversion = FEATURE.REPROG_CONTROLS_V2
         else:
-            if _log.isEnabledFor(_ERROR):
-                _log.error(f'Trying to read keys on device {device} which has no REPROG_CONTROLS(_VX) support.')
+            if logger.isEnabledFor(logging.ERROR):
+                logger.error(f'Trying to read keys on device {device} which has no REPROG_CONTROLS(_VX) support.')
             self.keyversion = None
         self.keys = [None] * count
 
@@ -620,8 +629,8 @@ class KeysArray:
                 self.cid_to_tid[cid] = tid
                 if group != 0:  # 0 = does not belong to a group
                     self.group_cids[special_keys.CID_GROUP[group]].append(cid)
-        elif _log.isEnabledFor(_WARNING):
-            _log.warn(f"Key with index {index} was expected to exist but device doesn't report it.")
+        elif logger.isEnabledFor(logging.WARNING):
+            logger.warn(f"Key with index {index} was expected to exist but device doesn't report it.")
 
     def _ensure_all_keys_queried(self):
         """The retrieval of key information is lazy, but for certain functionality
@@ -683,8 +692,8 @@ class KeysArrayV1(KeysArray):
             cid, tid, flags = _unpack('!HHB', keydata[:5])
             self.keys[index] = ReprogrammableKey(self.device, index, cid, tid, flags)
             self.cid_to_tid[cid] = tid
-        elif _log.isEnabledFor(_WARNING):
-            _log.warn(f"Key with index {index} was expected to exist but device doesn't report it.")
+        elif logger.isEnabledFor(logging.WARNING):
+            logger.warn(f"Key with index {index} was expected to exist but device doesn't report it.")
 
 
 class KeysArrayV4(KeysArrayV1):
@@ -703,8 +712,8 @@ class KeysArrayV4(KeysArrayV1):
             self.cid_to_tid[cid] = tid
             if group != 0:  # 0 = does not belong to a group
                 self.group_cids[special_keys.CID_GROUP[group]].append(cid)
-        elif _log.isEnabledFor(_WARNING):
-            _log.warn(f"Key with index {index} was expected to exist but device doesn't report it.")
+        elif logger.isEnabledFor(logging.WARNING):
+            logger.warn(f"Key with index {index} was expected to exist but device doesn't report it.")
 
 
 # we are only interested in the current host, so use 0xFF for the host throughout
@@ -748,8 +757,8 @@ class KeysArrayPersistent(KeysArray):
             elif actionId == special_keys.ACTIONID.Empty:  # purge data from empty value
                 remapped = modifiers = 0
             self.keys[index] = PersistentRemappableAction(self.device, index, key, actionId, remapped, modifiers, status)
-        elif _log.isEnabledFor(_WARNING):
-            _log.warn(f"Key with index {index} was expected to exist but device doesn't report it.")
+        elif logger.isEnabledFor(logging.WARNING):
+            logger.warn(f"Key with index {index} was expected to exist but device doesn't report it.")
 
 
 # Gesture Ids for feature GESTURE_2
@@ -1036,8 +1045,8 @@ class Spec:
         try:
             value = feature_request(self._device, FEATURE.GESTURE_2, 0x50, self.id, 0xFF)
         except FeatureCallError:  # some calls produce an error (notably spec 5 multiplier on K400Plus)
-            if _log.isEnabledFor(_WARNING):
-                _log.warn(f'Feature Call Error reading Gesture Spec on device {self._device} for spec {self.id} - use None')
+            if logger.isEnabledFor(logging.WARNING):
+                logger.warn(f'Feature Call Error reading Gesture Spec on device {self._device} for spec {self.id} - use None')
             return None
         return _bytes2int(value[:self.byte_count])
 
@@ -1079,12 +1088,12 @@ class Gestures:
                     self.params[param.param] = param
                 elif field_high == 0x04:
                     if field_low != 0x00:
-                        _log.error(f'Unimplemented GESTURE_2 grouping {field_low} {field_high} found.')
+                        logger.error(f'Unimplemented GESTURE_2 grouping {field_low} {field_high} found.')
                 elif field_high & 0xF0 == 0x40:
                     spec = Spec(device, field_low, field_high)
                     self.specs[spec.spec] = spec
                 else:
-                    _log.warn(f'Unimplemented GESTURE_2 field {field_low} {field_high} found.')
+                    logger.warn(f'Unimplemented GESTURE_2 field {field_low} {field_high} found.')
                 index += 1
 
     def gesture(self, gesture):
@@ -1114,6 +1123,482 @@ class Gestures:
         return g.set(self.device, value) if g else None
 
 
+class Backlight:
+    """Information about the current settings of x1982 Backlight2 v3, but also works for previous versions"""
+
+    def __init__(self, device):
+        response = device.feature_request(FEATURE.BACKLIGHT2, 0x00)
+        if not response:
+            raise FeatureCallError(msg='No reply from device.')
+        self.device = device
+        self.enabled, self.options, supported, effects, self.level, self.dho, self.dhi, self.dpow = _unpack(
+            '<BBBHBHHH', response[:12]
+        )
+        self.auto_supported = supported & 0x08
+        self.temp_supported = supported & 0x10
+        self.perm_supported = supported & 0x20
+        self.mode = (self.options >> 3) & 0x03
+
+    def write(self):
+        self.options = (self.options & 0x07) | (self.mode << 3)
+        level = self.level if self.mode == 0x3 else 0
+        data_bytes = _pack('<BBBBHHH', self.enabled, self.options, 0xFF, level, self.dho, self.dhi, self.dpow)
+        self.device.feature_request(FEATURE.BACKLIGHT2, 0x00)  # for testing - remove later
+        self.device.feature_request(FEATURE.BACKLIGHT2, 0x10, data_bytes)
+
+
+class LEDParam:
+    color = 'color'
+    speed = 'speed'
+    period = 'period'
+    intensity = 'intensity'
+    ramp = 'ramp'
+    form = 'form'
+
+
+LEDRampChoices = _NamedInts(default=0, yes=1, no=2)
+LEDFormChoices = _NamedInts(default=0, sine=1, square=2, triangle=3, sawtooth=4, sharkfin=5, exponential=6)
+LEDParamSize = {LEDParam.color: 3, LEDParam.speed: 1, LEDParam.period: 2,
+                LEDParam.intensity: 1, LEDParam.ramp: 1, LEDParam.form: 1}   # yapf: disable
+LEDEffects = {  # Wave=0x04, Stars=0x05, Press=0x06, Audio=0x07,   # not implemented
+    0x0: [_NamedInt(0x0, _('Disabled')), {}],
+    0x1: [_NamedInt(0x1, _('Static')), {LEDParam.color: 0, LEDParam.ramp: 3}],
+    0x2: [_NamedInt(0x2, _('Pulse')), {LEDParam.color: 0, LEDParam.speed: 3}],
+    0x3: [_NamedInt(0x3, _('Cycle')), {LEDParam.period: 5, LEDParam.intensity: 7}],
+    0x8: [_NamedInt(0x8, _('Boot')), {}],
+    0x9: [_NamedInt(0x9, _('Demo')), {}],
+    0xA: [_NamedInt(0xA, _('Breathe')), {LEDParam.color: 0, LEDParam.period: 3,
+                                         LEDParam.form: 5, LEDParam.intensity: 6}],
+    0xB: [_NamedInt(0xB, _('Ripple')), {LEDParam.color: 0, LEDParam.period: 4}]
+}   # yapf: disable
+
+
+class LEDEffectSetting:  # an effect plus its parameters
+
+    def __init__(self, **kwargs):
+        self.ID = None
+        for key, val in kwargs.items():
+            setattr(self, key, val)
+
+    @classmethod
+    def from_bytes(cls, bytes, options=None):
+        ID = next((ze.ID for ze in options if ze.index == bytes[0]), None) if options is not None else bytes[0]
+        effect = LEDEffects[ID] if ID in LEDEffects else None
+        args = {'ID': effect[0] if effect else None}
+        if effect:
+            for p, b in effect[1].items():
+                args[str(p)] = _bytes2int(bytes[1 + b:1 + b + LEDParamSize[p]])
+        else:
+            args['bytes'] = bytes
+        return cls(**args)
+
+    def to_bytes(self, options=None):
+        ID = self.ID
+        if ID is None:
+            return self.bytes if hasattr(self, 'bytes') else b'\xff' * 11
+        else:
+            bs = [0] * 10
+            for p, b in LEDEffects[ID][1].items():
+                bs[b:b + LEDParamSize[p]] = _int2bytes(getattr(self, str(p), 0), LEDParamSize[p])
+            if options is not None:
+                ID = next((ze.index for ze in options if ze.ID == ID), None)
+            result = _int2bytes(ID, 1) + bytes(bs)
+            return result
+
+    @classmethod
+    def from_yaml(cls, loader, node):
+        return cls(**loader.construct_mapping(node))
+
+    @classmethod
+    def to_yaml(cls, dumper, data):
+        return dumper.represent_mapping('!LEDEffectSetting', data.__dict__, flow_style=True)
+
+    def __str__(self):
+        return _yaml.dump(self, width=float('inf')).rstrip('\n')
+
+
+_yaml.SafeLoader.add_constructor('!LEDEffectSetting', LEDEffectSetting.from_yaml)
+_yaml.add_representer(LEDEffectSetting, LEDEffectSetting.to_yaml)
+
+
+class LEDEffectInfo:  # an effect that a zone can do
+
+    def __init__(self, device, zindex, eindex):
+        info = device.feature_request(FEATURE.COLOR_LED_EFFECTS, 0x20, zindex, eindex)
+        self.zindex, self.index, self.ID, self.capabilities, self.period = _unpack('!BBHHH', info[0:8])
+
+    def __str__(self):
+        return f'LEDEffectInfo({self.zindex}, {self.index}, {self.ID}, {self.capabilities: x}, {self.period})'
+
+
+LEDZoneLocations = _NamedInts()
+LEDZoneLocations[0x00] = _('Unknown Location')
+LEDZoneLocations[0x01] = _('Primary')
+LEDZoneLocations[0x02] = _('Logo')
+LEDZoneLocations[0x03] = _('Left Side')
+LEDZoneLocations[0x04] = _('Right Side')
+LEDZoneLocations[0x05] = _('Combined')
+LEDZoneLocations[0x06] = _('Primary 1')
+LEDZoneLocations[0x07] = _('Primary 2')
+LEDZoneLocations[0x08] = _('Primary 3')
+LEDZoneLocations[0x09] = _('Primary 4')
+LEDZoneLocations[0x0A] = _('Primary 5')
+LEDZoneLocations[0x0B] = _('Primary 6')
+
+
+class LEDZoneInfo:  # effects that a zone can do
+
+    def __init__(self, device, index):
+        info = device.feature_request(FEATURE.COLOR_LED_EFFECTS, 0x10, index)
+        self.index, self.location, self.count = _unpack('!BHB', info[0:4])
+        self.location = LEDZoneLocations[self.location] if LEDZoneLocations[self.location] else self.location
+        self.effects = []
+        for i in range(0, self.count):
+            self.effects.append(LEDEffectInfo(device, index, i))
+
+    def to_command(self, setting):
+        for i in range(0, len(self.effects)):
+            e = self.effects[i]
+            if e.ID == setting.ID:
+                return _int2bytes(self.index) + _int2bytes(i) + setting.to_bytes()[1:]
+        return None
+
+    def __str__(self):
+        return f'LEDZoneInfo({self.index}, {self.location}, {[str(z) for z in self.effects]}'
+
+
+class LEDEffectsInfo:  # effects that the LEDs can do
+
+    def __init__(self, device):
+        info = device.feature_request(FEATURE.COLOR_LED_EFFECTS, 0x00)
+        self.device = device
+        self.count, _, capabilities = _unpack('!BHH', info[0:5])
+        self.readable = capabilities & 0x1
+        self.zones = []
+        for i in range(0, self.count):
+            self.zones.append(LEDZoneInfo(device, i))
+
+    def to_command(self, index, setting):
+        return self.zones[index].to_command(setting)
+
+    def __str__(self):
+        zones = '\n'.join([str(z) for z in self.zones])
+        return f'LEDEffectsInfo({self.device}, readable {self.readable}\n{zones})'
+
+
+ButtonBehaviors = _NamedInts(MacroExecute=0x0, MacroStop=0x1, MacroStopAll=0x2, Send=0x8, Function=0x9)
+ButtonMappingTypes = _NamedInts(No_Action=0x0, Button=0x1, Modifier_And_Key=0x2, Consumer_Key=0x3)
+ButtonFunctions = _NamedInts(
+    No_Action=0x0,
+    Tilt_Left=0x1,
+    Tilt_Right=0x2,
+    Next_DPI=0x3,
+    Previous_DPI=0x4,
+    Cycle_DPI=0x5,
+    Default_DPI=0x6,
+    Shift_DPI=0x7,
+    Next_Profile=0x8,
+    Previous_Profile=0x9,
+    Cycle_Profile=0xA,
+    G_Shift=0xB,
+    Battery_Status=0xC,
+    Profile_Select=0xD,
+    Mode_Switch=0xE,
+    Host_Button=0xF,
+    Scroll_Down=0x10,
+    Scroll_Up=0x11
+)
+ButtonButtons = special_keys.MOUSE_BUTTONS
+ButtonModifiers = special_keys.modifiers
+ButtonKeys = special_keys.USB_HID_KEYCODES
+ButtonConsumerKeys = special_keys.HID_CONSUMERCODES
+
+
+class Button:
+    """A button mapping"""
+
+    def __init__(self, **kwargs):
+        self.behavior = None
+        for key, val in kwargs.items():
+            setattr(self, key, val)
+
+    @classmethod
+    def from_yaml(cls, loader, node):
+        args = loader.construct_mapping(node)
+        return cls(**args)
+
+    @classmethod
+    def to_yaml(cls, dumper, data):
+        return dumper.represent_mapping('!Button', data.__dict__, flow_style=True)
+
+    @classmethod
+    def from_bytes(cls, bytes):
+        behavior = ButtonBehaviors[bytes[0] >> 4]
+        if behavior == ButtonBehaviors.MacroExecute or behavior == ButtonBehaviors.MacroStop:
+            sector = (bytes[0] & 0x0F) << 8 + bytes[1]
+            address = bytes[2] << 8 + bytes[3]
+            result = cls(behavior=behavior, sector=sector, address=address)
+        elif behavior == ButtonBehaviors.Send:
+            mapping_type = ButtonMappingTypes[bytes[1]]
+            if mapping_type == ButtonMappingTypes.Button:
+                value = ButtonButtons[(bytes[2] << 8) + bytes[3]]
+                result = cls(behavior=behavior, type=mapping_type, value=value)
+            elif mapping_type == ButtonMappingTypes.Modifier_And_Key:
+                modifiers = bytes[2]
+                value = ButtonKeys[bytes[3]]
+                result = cls(behavior=behavior, type=mapping_type, modifiers=modifiers, value=value)
+            elif mapping_type == ButtonMappingTypes.Consumer_Key:
+                value = ButtonConsumerKeys[(bytes[2] << 8) + bytes[3]]
+                result = cls(behavior=behavior, type=mapping_type, value=value)
+            elif mapping_type == ButtonMappingTypes.No_Action:
+                result = cls(behavior=behavior, type=mapping_type)
+        elif behavior == ButtonBehaviors.Function:
+            value = ButtonFunctions[bytes[1]] if ButtonFunctions[bytes[1]] is not None else bytes[1]
+            data = bytes[3]
+            result = cls(behavior=behavior, value=value, data=data)
+        else:
+            result = cls(behavior=bytes[0] >> 4, bytes=bytes)
+        return result
+
+    def to_bytes(self):
+        bytes = _int2bytes(self.behavior << 4, 1) if self.behavior is not None else None
+        if self.behavior == ButtonBehaviors.MacroExecute or self.behavior == ButtonBehaviors.MacroStop:
+            bytes = _int2bytes(self.sector, 2) + _int2bytes(self.address, 2)
+            bytes[0] += self.behavior << 4
+        elif self.behavior == ButtonBehaviors.Send:
+            bytes += _int2bytes(self.type, 1)
+            if self.type == ButtonMappingTypes.Button:
+                bytes += _int2bytes(self.value, 2)
+            elif self.type == ButtonMappingTypes.Modifier_And_Key:
+                bytes += _int2bytes(self.modifiers, 1)
+                bytes += _int2bytes(self.value, 1)
+            elif self.type == ButtonMappingTypes.Consumer_Key:
+                bytes += _int2bytes(self.value, 2)
+            elif self.type == ButtonMappingTypes.No_Action:
+                bytes += b'\xff\xff'
+        elif self.behavior == ButtonBehaviors.Function:
+            bytes += _int2bytes(self.value, 1) + b'\xff' + (_int2bytes(self.data, 1) if self.data else b'\x00')
+        else:
+            bytes = self.bytes if self.bytes else b'\xff\xff\xff\xff'
+        return bytes
+
+    def __repr__(self):
+        return '%s{%s}' % (
+            self.__class__.__name__, ', '.join([str(key) + ':' + str(val) for key, val in self.__dict__.items()])
+        )
+
+
+_yaml.SafeLoader.add_constructor('!Button', Button.from_yaml)
+_yaml.add_representer(Button, Button.to_yaml)
+
+
+class OnboardProfile:
+    """A single onboard profile"""
+
+    def __init__(self, **kwargs):
+        for key, val in kwargs.items():
+            setattr(self, key, val)
+
+    @classmethod
+    def from_yaml(cls, loader, node):
+        args = loader.construct_mapping(node)
+        return cls(**args)
+
+    @classmethod
+    def to_yaml(cls, dumper, data):
+        return dumper.represent_mapping('!OnboardProfile', data.__dict__)
+
+    @classmethod
+    def from_bytes(cls, sector, enabled, buttons, gbuttons, bytes):
+        return cls(
+            sector=sector,
+            enabled=enabled,
+            report_rate=bytes[0],
+            resolution_default_index=bytes[1],
+            resolution_shift_index=bytes[2],
+            resolutions=[_unpack('<H', bytes[i * 2 + 3:i * 2 + 5])[0] for i in range(0, 5)],
+            red=bytes[13],
+            green=bytes[14],
+            blue=bytes[15],
+            power_mode=bytes[16],
+            angle_snap=bytes[17],
+            write_count=_unpack('<H', bytes[18:20])[0],
+            reserved=bytes[20:28],
+            ps_timeout=_unpack('<H', bytes[28:30])[0],
+            po_timeout=_unpack('<H', bytes[30:32])[0],
+            buttons=[Button.from_bytes(bytes[32 + i * 4:32 + i * 4 + 4]) for i in range(0, buttons)],
+            gbuttons=[Button.from_bytes(bytes[96 + i * 4:96 + i * 4 + 4]) for i in range(0, gbuttons)],
+            name=bytes[160:208].decode('utf-16le').rstrip('\x00').rstrip('\uFFFF'),
+            lighting=[LEDEffectSetting.from_bytes(bytes[208 + i * 11:219 + i * 11]) for i in range(0, 4)]
+        )
+
+    @classmethod
+    def from_dev(cls, dev, i, sector, s, enabled, buttons, gbuttons):
+        bytes = OnboardProfiles.read_sector(dev, sector, s)
+        return cls.from_bytes(sector, enabled, buttons, gbuttons, bytes)
+
+    def to_bytes(self, length):
+        bytes = _int2bytes(self.report_rate, 1)
+        bytes += _int2bytes(self.resolution_default_index, 1) + _int2bytes(self.resolution_shift_index, 1)
+        bytes += b''.join([self.resolutions[i].to_bytes(2, 'little') for i in range(0, 5)])
+        bytes += _int2bytes(self.red, 1) + _int2bytes(self.green, 1) + _int2bytes(self.blue, 1)
+        bytes += _int2bytes(self.power_mode, 1) + _int2bytes(self.angle_snap, 1)
+        bytes += self.write_count.to_bytes(2, 'little') + self.reserved
+        bytes += self.ps_timeout.to_bytes(2, 'little') + self.po_timeout.to_bytes(2, 'little')
+        for i in range(0, 16):
+            bytes += self.buttons[i].to_bytes() if i < len(self.buttons) else b'\xff\xff\xff\xff'
+        for i in range(0, 16):
+            bytes += self.gbuttons[i].to_bytes() if i < len(self.gbuttons) else b'\xff\xff\xff\xff'
+        if self.enabled:
+            bytes += self.name[0:24].ljust(24, '\x00').encode('utf-16le')
+        else:
+            bytes += b'\xff' * 48
+        for i in range(0, 4):
+            bytes += self.lighting[i].to_bytes()
+        while len(bytes) < length - 2:
+            bytes += b'\xff'
+        bytes += _int2bytes(_crc16(bytes), 2)
+        return bytes
+
+    def dump(self):
+        print(f'     Onboard Profile: {self.name}')
+        print(f'       Report Rate {self.report_rate} ms')
+        print(f'       DPI Resolutions {self.resolutions}')
+        print(f'       Default Resolution Index {self.res_index}, Shift Resolution Index {self.res_shift_index}')
+        print(f'       Colors {self.red} {self.green} {self.blue}')
+        print(f'       Power {self.power_mode}, Angle Snapping {self.angle_snap}')
+        for i in range(0, len(self.buttons)):
+            if self.buttons[i].behavior is not None:
+                print('       BUTTON', i + 1, self.buttons[i])
+        for i in range(0, len(self.gbuttons)):
+            if self.gbuttons[i].behavior is not None:
+                print('       G-BUTTON', i + 1, self.gbuttons[i])
+
+
+_yaml.SafeLoader.add_constructor('!OnboardProfile', OnboardProfile.from_yaml)
+_yaml.add_representer(OnboardProfile, OnboardProfile.to_yaml)
+
+OnboardProfilesVersion = 3
+
+
+# Doesn't handle macros
+class OnboardProfiles:
+    """The entire onboard profiles information"""
+
+    def __init__(self, **kwargs):
+        for key, val in kwargs.items():
+            setattr(self, key, val)
+
+    @classmethod
+    def from_yaml(cls, loader, node):
+        args = loader.construct_mapping(node)
+        return cls(**args)
+
+    @classmethod
+    def to_yaml(cls, dumper, data):
+        return dumper.represent_mapping('!OnboardProfiles', data.__dict__)
+
+    @classmethod
+    def get_profile_headers(cls, device):
+        i = 0
+        headers = []
+        chunk = device.feature_request(FEATURE.ONBOARD_PROFILES, 0x50, 0, 0, 0, i)
+        s = 0x00
+        if chunk[0:4] == b'\x00\x00\x00\x00':  # look in ROM instead
+            chunk = device.feature_request(FEATURE.ONBOARD_PROFILES, 0x50, 0x01, 0, 0, i)
+            s = 0x01
+        while chunk[0:2] != b'\xff\xff':
+            sector, enabled = _unpack('!HB', chunk[0:3])
+            headers.append((sector, enabled))
+            i += 1
+            chunk = device.feature_request(FEATURE.ONBOARD_PROFILES, 0x50, s, 0, 0, i * 4)
+        return headers
+
+    @classmethod
+    def from_device(cls, device):
+        if not device.online:  # wake the device up if necessary
+            device.ping()
+        response = device.feature_request(FEATURE.ONBOARD_PROFILES, 0x00)
+        memory, profile, _macro = _unpack('!BBB', response[0:3])
+        if memory != 0x01 or profile > 0x04:
+            return
+        count, oob, buttons, sectors, size, shift = _unpack('!BBBBHB', response[3:10])
+        gbuttons = buttons if (shift & 0x3 == 0x2) else 0
+        headers = OnboardProfiles.get_profile_headers(device)
+        profiles = {}
+        i = 0
+        for sector, enabled in headers:
+            profiles[i + 1] = OnboardProfile.from_dev(device, i, sector, size, enabled, buttons, gbuttons)
+            i += 1
+        return cls(
+            version=OnboardProfilesVersion,
+            name=device.name,
+            count=count,
+            buttons=buttons,
+            gbuttons=gbuttons,
+            sectors=sectors,
+            size=size,
+            profiles=profiles
+        )
+
+    def to_bytes(self):
+        bytes = b''
+        for i in range(1, len(self.profiles) + 1):
+            bytes += _int2bytes(self.profiles[i].sector, 2) + _int2bytes(self.profiles[i].enabled, 1) + b'\x00'
+        bytes += b'\xff\xff\x00\x00'  # marker after last profile
+        while len(bytes) < self.size - 2:  # leave room for CRC
+            bytes += b'\xff'
+        bytes += _int2bytes(_crc16(bytes), 2)
+        return bytes
+
+    @classmethod
+    def read_sector(cls, dev, sector, s):  # doesn't check for valid sector or size
+        bytes = b''
+        o = 0
+        while o < s - 15:
+            chunk = dev.feature_request(FEATURE.ONBOARD_PROFILES, 0x50, sector >> 8, sector & 0xFF, o >> 8, o & 0xFF)
+            bytes += chunk
+            o += 16
+        chunk = dev.feature_request(FEATURE.ONBOARD_PROFILES, 0x50, sector >> 8, sector & 0xFF, (s - 16) >> 8, (s - 16) & 0xFF)
+        bytes += chunk[16 + o - s:]  # the last chunk has to be read in an awkward way
+        return bytes
+
+    @classmethod
+    def write_sector(cls, device, s, bs):  # doesn't check for valid sector or size
+        rbs = OnboardProfiles.read_sector(device, s, len(bs))
+        if rbs[:-2] == bs[:-2]:
+            return False
+        device.feature_request(FEATURE.ONBOARD_PROFILES, 0x60, s >> 8, s & 0xFF, 0, 0, len(bs) >> 8, len(bs) & 0xFF)
+        o = 0
+        while o < len(bs) - 1:
+            device.feature_request(FEATURE.ONBOARD_PROFILES, 0x70, bs[o:o + 16])
+            o += 16
+        device.feature_request(FEATURE.ONBOARD_PROFILES, 0x80)
+        return True
+
+    def write(self, device):
+        try:
+            written = 1 if OnboardProfiles.write_sector(device, 0, self.to_bytes()) else 0
+        except Exception as e:
+            logger.warn('Exception writing onboard profile control sector')
+            raise e
+        for p in self.profiles.values():
+            try:
+                if p.sector >= self.sectors:
+                    raise Exception(f'Sector {p.sector} not a writable sector')
+                written += 1 if OnboardProfiles.write_sector(device, p.sector, p.to_bytes(self.size)) else 0
+            except Exception as e:
+                logger.warn(f'Exception writing onboard profile sector {p.sector}')
+                raise e
+        return written
+
+    def show(self):
+        print(_yaml.dump(self))
+
+
+_yaml.SafeLoader.add_constructor('!OnboardProfiles', OnboardProfiles.from_yaml)
+_yaml.add_representer(OnboardProfiles, OnboardProfiles.to_yaml)
+
 #
 #
 #
@@ -1153,8 +1638,8 @@ def get_firmware(device):
                     fw_info = _FirmwareInfo(FIRMWARE_KIND.Other, '', '', None)
 
                 fw.append(fw_info)
-                # if _log.isEnabledFor(_DEBUG):
-                #     _log.debug("device %d firmware %s", devnumber, fw_info)
+                # if logger.isEnabledFor(logging.DEBUG):
+                #     logger.debug("device %d firmware %s", devnumber, fw_info)
         return tuple(fw)
 
 
@@ -1184,8 +1669,8 @@ def get_kind(device):
     kind = feature_request(device, FEATURE.DEVICE_NAME, 0x20)
     if kind:
         kind = ord(kind[:1])
-        # if _log.isEnabledFor(_DEBUG):
-        #     _log.debug("device %d type %d = %s", devnumber, kind, DEVICE_KIND[kind])
+        # if logger.isEnabledFor(logging.DEBUG):
+        #     logger.debug("device %d type %d = %s", devnumber, kind, DEVICE_KIND[kind])
         return DEVICE_KIND[kind]
 
 
@@ -1205,7 +1690,7 @@ def get_name(device):
             if fragment:
                 name += fragment[:name_length - len(name)]
             else:
-                _log.error('failed to read whole name of %s (expected %d chars)', device, name_length)
+                logger.error('failed to read whole name of %s (expected %d chars)', device, name_length)
                 return None
 
         return name.decode('utf-8')
@@ -1228,7 +1713,7 @@ def get_friendly_name(device):
                 initial_null = 0 if fragment[0] else 1  # initial null actually seen on a device
                 name += fragment[initial_null:name_length + initial_null - len(name)]
             else:
-                _log.error('failed to read whole name of %s (expected %d chars)', device, name_length)
+                logger.error('failed to read whole name of %s (expected %d chars)', device, name_length)
                 return None
 
         return name.decode('utf-8')
@@ -1244,8 +1729,8 @@ def decipher_battery_status(report):
     discharge, next, status = _unpack('!BBB', report[:3])
     discharge = None if discharge == 0 else discharge
     status = BATTERY_STATUS[status]
-    if _log.isEnabledFor(_DEBUG):
-        _log.debug('battery status %s%% charged, next %s%%, status %s', discharge, next, status)
+    if logger.isEnabledFor(logging.DEBUG):
+        logger.debug('battery status %s%% charged, next %s%%, status %s', discharge, next, status)
     return FEATURE.BATTERY_STATUS, discharge, next, status, None
 
 
@@ -1258,8 +1743,8 @@ def get_battery_unified(device):
 def decipher_battery_unified(report):
     discharge, level, status, _ignore = _unpack('!BBBB', report[:4])
     status = BATTERY_STATUS[status]
-    if _log.isEnabledFor(_DEBUG):
-        _log.debug('battery unified %s%% charged, level %s, charging %s', discharge, level, status)
+    if logger.isEnabledFor(logging.DEBUG):
+        logger.debug('battery unified %s%% charged, level %s, charging %s', discharge, level, status)
     level = (
         _BATTERY_APPROX.full if level == 8  # full
         else _BATTERY_APPROX.good if level == 4  # good
@@ -1320,8 +1805,8 @@ def decipher_battery_voltage(report):
         if level[0] < voltage:
             charge_lvl = level[1]
             break
-    if _log.isEnabledFor(_DEBUG):
-        _log.debug(
+    if logger.isEnabledFor(logging.DEBUG):
+        logger.debug(
             'battery voltage %d mV, charging %s, status %d = %s, level %s, type %s', voltage, status, (flags & 0x03),
             charge_sts, charge_lvl, charge_type
         )
@@ -1399,6 +1884,20 @@ def get_gestures(device):
         return Gestures(device)
 
 
+def get_backlight(device):
+    if getattr(device, '_backlight', None) is not None:
+        return device._backlight
+    if FEATURE.BACKLIGHT2 in device.features:
+        return Backlight(device)
+
+
+def get_profiles(device):
+    if getattr(device, '_profiles', None) is not None:
+        return device._profiles
+    if FEATURE.ONBOARD_PROFILES in device.features:
+        return OnboardProfiles.from_device(device)
+
+
 def get_mouse_pointer_info(device):
     pointer_info = feature_request(device, FEATURE.MOUSE_POINTER)
     if pointer_info:
@@ -1515,8 +2014,8 @@ def get_host_names(device):
 def set_host_name(device, name, currentName=''):
     name = bytearray(name, 'utf-8')
     currentName = bytearray(currentName, 'utf-8')
-    if _log.isEnabledFor(_INFO):
-        _log.info('Setting host name to %s', name)
+    if logger.isEnabledFor(logging.INFO):
+        logger.info('Setting host name to %s', name)
     state = feature_request(device, FEATURE.HOSTS_INFO, 0x00)
     if state:
         flags, _ignore, _ignore, currentHost = _unpack('!BBBB', state[:4])
@@ -1552,7 +2051,13 @@ def get_polling_rate(device):
     state = feature_request(device, FEATURE.REPORT_RATE, 0x10)
     if state:
         rate = _unpack('!B', state[:1])[0]
-        return rate
+        return str(rate) + 'ms'
+    else:
+        rates = ['8ms', '4ms', '2ms', '1ms', '500us', '250us', '125us']
+        state = feature_request(device, FEATURE.EXTENDED_ADJUSTABLE_REPORT_RATE, 0x20)
+        if state:
+            rate = _unpack('!B', state[:1])[0]
+            return rates[rate]
 
 
 def get_remaining_pairing(device):
