@@ -16,12 +16,9 @@
 ## with this program; if not, write to the Free Software Foundation, Inc.,
 ## 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 
+import logging
 import threading as _threading
 
-from logging import DEBUG as _DEBUG
-from logging import INFO as _INFO
-from logging import getLogger
-
 from . import base as _base
 
 # from time import time as _timestamp
@@ -32,8 +29,7 @@ try:
 except ImportError:
     from queue import Queue as _Queue
 
-_log = getLogger(__name__)
-del getLogger
+logger = logging.getLogger(__name__)
 
 #
 #
@@ -64,10 +60,10 @@ class _ThreadedHandle:
     def _open(self):
         handle = _base.open_path(self.path)
         if handle is None:
-            _log.error('%r failed to open new handle', self)
+            logger.error('%r failed to open new handle', self)
         else:
-            # if _log.isEnabledFor(_DEBUG):
-            #     _log.debug("%r opened new handle %d", self, handle)
+            # if logger.isEnabledFor(logging.DEBUG):
+            #     logger.debug("%r opened new handle %d", self, handle)
             self._local.handle = handle
             self._handles.append(handle)
             return handle
@@ -76,8 +72,8 @@ class _ThreadedHandle:
         if self._local:
             self._local = None
             handles, self._handles = self._handles, []
-            if _log.isEnabledFor(_DEBUG):
-                _log.debug('%r closing %s', self, handles)
+            if logger.isEnabledFor(logging.DEBUG):
+                logger.debug('%r closing %s', self, handles)
             for h in handles:
                 _base.close(h)
 
@@ -140,66 +136,46 @@ class EventsListener(_threading.Thread):
     """
 
     def __init__(self, receiver, notifications_callback):
-        super().__init__(name=self.__class__.__name__ + ':' + receiver.path.split('/')[2])
-
+        try:
+            path_name = receiver.path.split('/')[2]
+        except IndexError:
+            path_name = receiver.path
+        super().__init__(name=self.__class__.__name__ + ':' + path_name)
         self.daemon = True
         self._active = False
-
         self.receiver = receiver
         self._queued_notifications = _Queue(16)
         self._notifications_callback = notifications_callback
 
-        # self.tick_period = 0
-
     def run(self):
         self._active = True
-
         # replace the handle with a threaded one
         self.receiver.handle = _ThreadedHandle(self, self.receiver.path, self.receiver.handle)
-        # get the right low-level handle for this thread
-        ihandle = int(self.receiver.handle)
-        if _log.isEnabledFor(_INFO):
-            _log.info('started with %s (%d)', self.receiver, ihandle)
-
+        if logger.isEnabledFor(logging.INFO):
+            logger.info('started with %s (%d)', self.receiver, int(self.receiver.handle))
         self.has_started()
 
-        # last_tick = 0
-        # the first idle read -- delay it a bit, and make sure to stagger
-        # idle reads for multiple receivers
-        # idle_reads = _IDLE_READS + (ihandle % 5) * 2
+        if self.receiver.isDevice:  # ping (wired or BT) devices to see if they are really online
+            if self.receiver.ping():
+                self.receiver.status.changed(True, reason='initialization')
 
         while self._active:
             if self._queued_notifications.empty():
                 try:
-                    # _log.debug("read next notification")
                     n = _base.read(self.receiver.handle, _EVENT_READ_TIMEOUT)
                 except _base.NoReceiver:
-                    _log.warning('%s disconnected', self.receiver.name)
+                    logger.warning('%s disconnected', self.receiver.name)
                     self.receiver.close()
                     break
-
                 if n:
                     n = _base.make_notification(*n)
             else:
-                # deliver any queued notifications
-                n = self._queued_notifications.get()
-
+                n = self._queued_notifications.get()  # deliver any queued notifications
             if n:
-                # if _log.isEnabledFor(_DEBUG):
-                #     _log.debug("%s: processing %s", self.receiver, n)
                 try:
                     self._notifications_callback(n)
                 except Exception:
-                    _log.exception('processing %s', n)
-
-            # elif self.tick_period:
-            #     idle_reads -= 1
-            #     if idle_reads <= 0:
-            #         idle_reads = _IDLE_READS
-            #         now = _timestamp()
-            #         if now - last_tick >= self.tick_period:
-            #             last_tick = now
-            #             self.tick(now)
+                    logger.exception('processing %s', n)
 
         del self._queued_notifications
         self.has_stopped()
@@ -226,8 +202,8 @@ class EventsListener(_threading.Thread):
         # i.e. triggered by a callback handling a previous notification.
         assert _threading.current_thread() == self
         if self._active:  # and _threading.current_thread() == self:
-            # if _log.isEnabledFor(_DEBUG):
-            #     _log.debug("queueing unhandled %s", n)
+            # if logger.isEnabledFor(logging.DEBUG):
+            #     logger.debug("queueing unhandled %s", n)
             if not self._queued_notifications.full():
                 self._queued_notifications.put(n)
 
