@@ -16,10 +16,9 @@
 ## with this program; if not, write to the Free Software Foundation, Inc.,
 ## 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 
-from logging import DEBUG as _DEBUG
-from logging import INFO as _INFO
+import logging
+
 from logging import WARN as _WARN
-from logging import getLogger
 from struct import pack as _pack
 from struct import unpack as _unpack
 from time import time as _time
@@ -32,6 +31,7 @@ from .common import NamedInts as _NamedInts
 from .common import bytes2int as _bytes2int
 from .common import int2bytes as _int2bytes
 from .i18n import _
+from .settings import KIND as _KIND
 from .settings import ActionSettingRW as _ActionSettingRW
 from .settings import BitFieldSetting as _BitFieldSetting
 from .settings import BitFieldValidator as _BitFieldV
@@ -40,6 +40,7 @@ from .settings import BitFieldWithOffsetAndMaskValidator as _BitFieldOMV
 from .settings import ChoicesMapValidator as _ChoicesMapV
 from .settings import ChoicesValidator as _ChoicesV
 from .settings import FeatureRW as _FeatureRW
+from .settings import HeteroValidator as _HeteroV
 from .settings import LongSettings as _LongSettings
 from .settings import MultipleRangeValidator as _MultipleRangeV
 from .settings import PackedRangeValidator as _PackedRangeV
@@ -50,8 +51,7 @@ from .settings import Setting as _Setting
 from .settings import Settings as _Settings
 from .special_keys import DISABLE as _DKEY
 
-_log = getLogger(__name__)
-del getLogger
+logger = logging.getLogger(__name__)
 
 _DK = _hidpp10.DEVICE_KIND
 _R = _hidpp10.REGISTERS
@@ -213,11 +213,138 @@ class Backlight(_Setting):
     validator_options = {'choices': choices_universe}
 
 
+# MX Keys S requires some extra values, as in 11 02 0c1a 000dff000b000b003c00000000000000
+# on/off options (from current) effect (FF-no change) level (from current) durations[6] (from current)
 class Backlight2(_Setting):
     name = 'backlight'
     label = _('Backlight')
-    description = _('Turn illumination on or off on keyboard.')
+    description = _('Illumination level on keyboard.  Changes made are only applied in Manual mode.')
     feature = _F.BACKLIGHT2
+    min_version = 0
+
+    class rw_class:
+
+        def __init__(self, feature):
+            self.feature = feature
+            self.kind = _FeatureRW.kind
+
+        def read(self, device):
+            backlight = device.backlight
+            if not backlight.enabled:
+                return 0xFF
+            else:
+                return backlight.mode
+
+        def write(self, device, data_bytes):
+            backlight = device.backlight
+            backlight.enabled = data_bytes[0] != 0xFF
+            if data_bytes[0] != 0xFF:
+                backlight.mode = data_bytes[0]
+            backlight.write()
+            return True
+
+    class validator_class(_ChoicesV):
+
+        @classmethod
+        def build(cls, setting_class, device):
+            backlight = device.backlight
+            choices = _NamedInts()
+            choices[0xFF] = _('Disabled')
+            if backlight.auto_supported:
+                choices[0x1] = _('Automatic')
+            if backlight.perm_supported:
+                choices[0x3] = _('Manual')
+            if not (backlight.auto_supported or backlight.temp_supported or backlight.perm_supported):
+                choices[0x0] = _('Enabled')
+            return cls(choices=choices, byte_count=1)
+
+
+class Backlight2Level(_Setting):
+    name = 'backlight_level'
+    label = _('Backlight Level')
+    description = _('Illumination level on keyboard when in Manual mode.')
+    feature = _F.BACKLIGHT2
+    min_version = 3
+
+    class rw_class:
+
+        def __init__(self, feature):
+            self.feature = feature
+            self.kind = _FeatureRW.kind
+
+        def read(self, device):
+            backlight = device.backlight
+            return _int2bytes(backlight.level, 1)
+
+        def write(self, device, data_bytes):
+            if device.backlight.level != _bytes2int(data_bytes):
+                device.backlight.level = _bytes2int(data_bytes)
+                device.backlight.write()
+            return True
+
+    class validator_class(_RangeV):
+
+        @classmethod
+        def build(cls, setting_class, device):
+            reply = device.feature_request(_F.BACKLIGHT2, 0x20)
+            assert reply, 'Oops, backlight range cannot be retrieved!'
+            if reply[0] > 1:
+                return cls(min_value=0, max_value=reply[0] - 1, byte_count=1)
+
+
+class Backlight2Duration(_Setting):
+    feature = _F.BACKLIGHT2
+    min_version = 3
+    validator_class = _RangeV
+    min_value = 1
+    max_value = 120  # actual maximum is 2 hours
+    validator_options = {'byte_count': 2}
+
+    class rw_class:
+
+        def __init__(self, feature, field):
+            self.feature = feature
+            self.kind = _FeatureRW.kind
+            self.field = field
+
+        def read(self, device):
+            backlight = device.backlight
+            return _int2bytes(getattr(backlight, self.field), 2) * 5  # use seconds instead of 5-second units
+
+        def write(self, device, data_bytes):
+            backlight = device.backlight
+            new_duration = (int.from_bytes(data_bytes) + 4) // 5  # use ceiling in 5-second units
+            if new_duration != getattr(backlight, self.field):
+                setattr(backlight, self.field, new_duration)
+                backlight.write()
+            return True
+
+
+class Backlight2DurationHandsOut(Backlight2Duration):
+    name = 'backlight_duration_hands_out'
+    label = _('Backlight Delay Hands Out')
+    description = _('Delay in seconds until backlight fades out with hands away from keyboard.')
+    feature = _F.BACKLIGHT2
+    validator_class = _RangeV
+    rw_options = {'field': 'dho'}
+
+
+class Backlight2DurationHandsIn(Backlight2Duration):
+    name = 'backlight_duration_hands_in'
+    label = _('Backlight Delay Hands In')
+    description = _('Delay in seconds until backlight fades out with hands near keyboard.')
+    feature = _F.BACKLIGHT2
+    validator_class = _RangeV
+    rw_options = {'field': 'dhi'}
+
+
+class Backlight2DurationPowered(Backlight2Duration):
+    name = 'backlight_duration_powered'
+    label = _('Backlight Delay Powered')
+    description = _('Delay in seconds until backlight fades out with external power.')
+    feature = _F.BACKLIGHT2
+    validator_class = _RangeV
+    rw_options = {'field': 'dpow'}
 
 
 class Backlight3(_Setting):
@@ -317,24 +444,67 @@ class ThumbInvert(_Setting):
 class OnboardProfiles(_Setting):
     name = 'onboard_profiles'
     label = _('Onboard Profiles')
-    description = _('Enable onboard profiles, which often control report rate and keyboard lighting')
+    description = _('Enable an onboard profile, which controls report rate, sensitivity, and button actions')
     feature = _F.ONBOARD_PROFILES
-    rw_options = {'read_fnid': 0x20, 'write_fnid': 0x10}
-    choices_universe = _NamedInts(Enable=1, Disable=2)
+    choices_universe = _NamedInts(Disabled=0)
+    for i in range(1, 6):
+        choices_universe[i] = f'Profile {i}'
     validator_class = _ChoicesV
-    validator_options = {'choices': choices_universe}
+
+    class rw_class:
+
+        def __init__(self, feature):
+            self.feature = feature
+            self.kind = _FeatureRW.kind
+
+        def read(self, device):
+            enabled = device.feature_request(_F.ONBOARD_PROFILES, 0x20)[0]
+            if enabled == 0x01:
+                active = device.feature_request(_F.ONBOARD_PROFILES, 0x40)
+                return active[:2]
+            else:
+                return b'\x00\x00'
+
+        def write(self, device, data_bytes):
+            from . import notifications as _notifications  # prevent circular import
+            if data_bytes == b'\x00\x00':
+                result = device.feature_request(_F.ONBOARD_PROFILES, 0x10, b'\x02')
+            else:
+                device.feature_request(_F.ONBOARD_PROFILES, 0x10, b'\x01')
+                result = device.feature_request(_F.ONBOARD_PROFILES, 0x30, data_bytes)
+                _notifications.profile_change(device, _bytes2int(data_bytes))
+            return result
+
+    class validator_class(_ChoicesV):
+
+        @classmethod
+        def build(cls, setting_class, device):
+            headers = _hidpp20.OnboardProfiles.get_profile_headers(device)
+            profiles_list = [setting_class.choices_universe[0]]
+            if headers:
+                for (sector, enabled) in headers:
+                    if enabled:
+                        profiles_list.append(setting_class.choices_universe[sector])
+            return cls(choices=_NamedInts.list(profiles_list), byte_count=2) if len(profiles_list) > 1 else None
 
 
 class ReportRate(_Setting):
     name = 'report_rate'
-    label = _('Polling Rate (ms)')
+    label = _('Report Rate')
     description = (
-        _('Frequency of device polling, in milliseconds') + '\n' +
-        _('May need Onboard Profiles set to Disable to be effective.')
+        _('Frequency of device movement reports') + '\n' + _('May need Onboard Profiles set to Disable to be effective.')
     )
     feature = _F.REPORT_RATE
     rw_options = {'read_fnid': 0x10, 'write_fnid': 0x20}
-    choices_universe = _NamedInts.range(1, 8)
+    choices_universe = _NamedInts()
+    choices_universe[1] = '1ms'
+    choices_universe[2] = '2ms'
+    choices_universe[3] = '3ms'
+    choices_universe[4] = '4ms'
+    choices_universe[5] = '5ms'
+    choices_universe[6] = '6ms'
+    choices_universe[7] = '7ms'
+    choices_universe[8] = '8ms'
 
     class _rw_class(_FeatureRW):  # no longer needed - set Onboard Profiles to disable
 
@@ -356,7 +526,51 @@ class ReportRate(_Setting):
             rate_flags = _bytes2int(reply[0:1])
             for i in range(0, 8):
                 if (rate_flags >> i) & 0x01:
-                    rate_list.append(i + 1)
+                    rate_list.append(setting_class.choices_universe[i + 1])
+            return cls(choices=_NamedInts.list(rate_list), byte_count=1) if rate_list else None
+
+
+class ExtendedReportRate(_Setting):
+    name = 'report_rate_extended'
+    label = _('Report Rate')
+    description = (
+        _('Frequency of device movement reports') + '\n' + _('May need Onboard Profiles set to Disable to be effective.')
+    )
+    feature = _F.EXTENDED_ADJUSTABLE_REPORT_RATE
+    rw_options = {'read_fnid': 0x20, 'write_fnid': 0x30}
+    choices_universe = _NamedInts()
+    choices_universe[0] = '8ms'
+    choices_universe[1] = '4ms'
+    choices_universe[2] = '2ms'
+    choices_universe[3] = '1ms'
+    choices_universe[4] = '500us'
+    choices_universe[5] = '250us'
+    choices_universe[6] = '125us'
+
+    class _rw_class(_FeatureRW):
+
+        def read(self, device, data_bytes=b''):
+            # need connection type from device to get actual report rate
+            self.read_prefix = b'\x00' if device.receiver else b'\x01'
+            super().read(device, data_bytes)
+
+        def write(self, device, data_bytes):
+            # Host mode is required for report rate to be adjustable
+            if _hidpp20.get_onboard_mode(device) != _hidpp20.ONBOARD_MODES.MODE_HOST:
+                _hidpp20.set_onboard_mode(device, _hidpp20.ONBOARD_MODES.MODE_HOST)
+            return super().write(device, data_bytes)
+
+    class validator_class(_ChoicesV):
+
+        @classmethod
+        def build(cls, setting_class, device):
+            reply = device.feature_request(_F.EXTENDED_ADJUSTABLE_REPORT_RATE, 0x10)
+            assert reply, 'Oops, report rate choices cannot be retrieved!'
+            rate_list = []
+            rate_flags = _bytes2int(reply[0:2])
+            for i in range(0, 6):
+                if (rate_flags & (0x01 << i)):
+                    rate_list.append(setting_class.choices_universe[i])
             return cls(choices=_NamedInts.list(rate_list), byte_count=1) if rate_list else None
 
 
@@ -380,11 +594,8 @@ class CrownSmooth(_Setting):
 
 class DivertGkeys(_Setting):
     name = 'divert-gkeys'
-    label = _('Divert G Keys')
-    description = (
-        _('Make G keys send GKEY HID++ notifications (which trigger Solaar rules but are otherwise ignored).') + '\n' +
-        _('May also make M keys and MR key send HID++ notifications')
-    )
+    label = _('Divert G and M Keys')
+    description = (_('Make G and M keys send HID++ notifications (which trigger Solaar rules but are otherwise ignored).'))
     feature = _F.GKEY
     validator_options = {'true_value': 0x01, 'false_value': 0x00, 'mask': 0xff}
 
@@ -590,8 +801,8 @@ class MouseGesturesXY(_RawXYProcessing):
             from .base import _HIDPP_Notification as _HIDPP_Notification
             from .diversion import process_notification as _process_notification
             self.push_mouse_event()
-            if _log.isEnabledFor(_INFO):
-                _log.info('mouse gesture notification %s', self.data)
+            if logger.isEnabledFor(logging.INFO):
+                logger.info('mouse gesture notification %s', self.data)
             payload = _pack('!' + (len(self.data) * 'h'), *self.data)
             notification = _HIDPP_Notification(0, 0, 0, 0, payload)
             _process_notification(self.device, self.device.status, notification, _hidpp20.FEATURE.MOUSE_GESTURE)
@@ -617,8 +828,8 @@ class MouseGesturesXY(_RawXYProcessing):
         self.data.append(1)
         self.data.append(key)
         self.lastEv = _time() * 1000  # _time_ns() / 1e6
-        if _log.isEnabledFor(_DEBUG):
-            _log.debug('mouse gesture key event %d %s', key, self.data)
+        if logger.isEnabledFor(logging.DEBUG):
+            logger.debug('mouse gesture key event %d %s', key, self.data)
 
     def push_mouse_event(self):
         x = int(self.dx)
@@ -630,8 +841,8 @@ class MouseGesturesXY(_RawXYProcessing):
         self.data.append(y)
         self.dx = 0.
         self.dy = 0.
-        if _log.isEnabledFor(_DEBUG):
-            _log.debug('mouse gesture move event %d %d %s', x, y, self.data)
+        if logger.isEnabledFor(logging.DEBUG):
+            logger.debug('mouse gesture move event %d %d %s', x, y, self.data)
 
 
 class DivertKeys(_Settings):
@@ -769,7 +980,7 @@ class SpeedChange(_Setting):
                 if speed_setting:
                     speed_setting.write(newSpeed)
                 else:
-                    _log.error('cannot save sensitivity setting on %s', self.device)
+                    logger.error('cannot save sensitivity setting on %s', self.device)
                 from solaar.ui import status_changed as _status_changed
                 _status_changed(self.device, refresh=True)  # update main window
             if self.device.persister:
@@ -1130,8 +1341,8 @@ class PersistentRemappableAction(_Settings):
             elif capabilities & 0x0023 == 0x0023:  # Key, Mouse, and HScroll Codes
                 keys = _special_keys.KEYS_KEYS_MOUSE_HSCROLL
             else:
-                if _log.isEnabledFor(_WARN):
-                    _log.warn('%s: unimplemented Persistent Remappable capability %s', device.name, hex(capabilities))
+                if logger.isEnabledFor(_WARN):
+                    logger.warning('%s: unimplemented Persistent Remappable capability %s', device.name, hex(capabilities))
                 return None
             choices = {}
             for k in remap_keys:
@@ -1146,8 +1357,8 @@ class PersistentRemappableAction(_Settings):
             reply_value = _bytes2int(reply_bytes[start:end]) & self.mask
             # Craft keyboard has a value that isn't valid so fudge these values
             if reply_value not in self.choices[key]:
-                if _log.isEnabledFor(_WARN):
-                    _log.warn('unusual persistent remappable action mapping %x: use Default', reply_value)
+                if logger.isEnabledFor(_WARN):
+                    logger.warning('unusual persistent remappable action mapping %x: use Default', reply_value)
                 reply_value = _special_keys.KEYS_Default
             return reply_value
 
@@ -1204,6 +1415,54 @@ class ADCPower(_Setting):
     validator_options = {'byte_count': 1}
 
 
+class LEDControl(_Setting):
+    name = 'led_control'
+    label = _('LED Control')
+    description = _('Switch control of LEDs between device and Solaar')
+    feature = _F.COLOR_LED_EFFECTS
+    rw_options = {'read_fnid': 0x70, 'write_fnid': 0x80}
+    choices_universe = _NamedInts(Device=0, Solaar=1)
+    validator_class = _ChoicesV
+    validator_options = {'choices': choices_universe}
+
+
+colors = _special_keys.COLORS
+_LEDP = _hidpp20.LEDParam
+
+
+# an LED Zone has an index, a set of possible LED effects, and an LED effect setting
+class LEDZoneSetting(_Setting):
+    name = 'led_zone_'
+    label = _('LED Zone Effects')
+    description = _('Set effect for LED Zone')
+    feature = _F.COLOR_LED_EFFECTS
+    color_field = {'name': _LEDP.color, 'kind': _KIND.choice, 'label': None, 'choices': colors}
+    speed_field = {'name': _LEDP.speed, 'kind': _KIND.range, 'label': _('Speed'), 'min': 0, 'max': 255}
+    period_field = {'name': _LEDP.period, 'kind': _KIND.range, 'label': _('Period'), 'min': 100, 'max': 5000}
+    intensity_field = {'name': _LEDP.intensity, 'kind': _KIND.range, 'label': _('Intensity'), 'min': 0, 'max': 100}
+    ramp_field = {'name': _LEDP.ramp, 'kind': _KIND.choice, 'label': _('Ramp'), 'choices': _hidpp20.LEDRampChoices}
+    #    form_field = { 'name': _LEDP.form, 'kind': _KIND.choice, 'label': _('Form'), 'choices': _hidpp20.LEDFormChoices }
+    possible_fields = [color_field, speed_field, period_field, intensity_field, ramp_field]
+
+    @classmethod
+    def build(cls, device):
+        infos = device.led_effects
+        settings = []
+        for zone in infos.zones:
+            prefix = _int2bytes(zone.index, 1)
+            rw = _FeatureRW(_F.COLOR_LED_EFFECTS, read_fnid=0xE0, write_fnid=0x30, prefix=prefix)
+            validator = _HeteroV(data_class=_hidpp20.LEDEffectSetting, options=zone.effects, readable=infos.readable)
+            setting = cls(device, rw, validator)
+            setting.name = cls.name + str(int(zone.location))
+            setting.label = _('LEDs') + ' ' + str(_hidpp20.LEDZoneLocations[zone.location])
+            choices = [_hidpp20.LEDEffects[e.ID][0] for e in zone.effects]
+            ID_field = {'name': 'ID', 'kind': _KIND.choice, 'label': None, 'choices': choices}
+            setting.possible_fields = [ID_field] + cls.possible_fields
+            setting.fields_map = _hidpp20.LEDEffects
+            settings.append(setting)
+        return settings
+
+
 SETTINGS = [
     RegisterHandDetection,  # simple
     RegisterSmoothScroll,  # simple
@@ -1222,12 +1481,19 @@ SETTINGS = [
     ThumbMode,  # working
     OnboardProfiles,
     ReportRate,  # working
+    ExtendedReportRate,
     PointerSpeed,  # simple
     AdjustableDpi,  # working
     SpeedChange,
     #    Backlight,  # not working - disabled temporarily
     Backlight2,  # working
+    Backlight2Level,
+    Backlight2DurationHandsOut,
+    Backlight2DurationHandsIn,
+    Backlight2DurationPowered,
     Backlight3,
+    LEDControl,
+    LEDZoneSetting,
     FnSwap,  # simple
     NewFnSwap,  # simple
     K375sFnSwap,  # working
@@ -1259,14 +1525,16 @@ SETTINGS = [
 def check_feature(device, sclass):
     if sclass.feature not in device.features:
         return
+    if sclass.min_version > device.features.get_feature_version(sclass.feature):
+        return
     try:
         detected = sclass.build(device)
-        if _log.isEnabledFor(_DEBUG):
-            _log.debug('check_feature %s [%s] detected %s', sclass.name, sclass.feature, detected)
+        if logger.isEnabledFor(logging.DEBUG):
+            logger.debug('check_feature %s [%s] detected %s', sclass.name, sclass.feature, detected)
         return detected
     except Exception as e:
         from traceback import format_exc
-        _log.error('check_feature %s [%s] error %s\n%s', sclass.name, sclass.feature, e, format_exc())
+        logger.error('check_feature %s [%s] error %s\n%s', sclass.name, sclass.feature, e, format_exc())
         return False  # differentiate from an error-free determination that the setting is not supported
 
 
@@ -1284,7 +1552,12 @@ def check_feature_settings(device, already_known):
             known_present = device.persister and sclass.name in device.persister
             if not any(s.name == sclass.name for s in already_known) and (known_present or sclass.name not in absent):
                 setting = check_feature(device, sclass)
-                if setting:
+                if isinstance(setting, list):
+                    for s in setting:
+                        already_known.append(s)
+                    if sclass.name in newAbsent:
+                        newAbsent.remove(sclass.name)
+                elif setting:
                     already_known.append(setting)
                     if sclass.name in newAbsent:
                         newAbsent.remove(sclass.name)
