@@ -16,20 +16,19 @@
 ## with this program; if not, write to the Free Software Foundation, Inc.,
 ## 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 
+import logging
 import traceback
 
-from logging import WARNING as _WARNING
-from logging import getLogger
 from threading import Timer as _Timer
 
 from gi.repository import Gdk, GLib, Gtk
+from logitech_receiver.hidpp20 import LEDEffectSetting as _LEDEffectSetting
 from logitech_receiver.settings import KIND as _SETTING_KIND
 from logitech_receiver.settings import SENSITIVITY_IGNORE as _SENSITIVITY_IGNORE
 from solaar.i18n import _, ngettext
 from solaar.ui import ui_async as _ui_async
 
-_log = getLogger(__name__)
-del getLogger
+logger = logging.getLogger(__name__)
 
 #
 #
@@ -40,7 +39,7 @@ def _read_async(setting, force_read, sbox, device_is_online, sensitive):
 
     def _do_read(s, force, sb, online, sensitive):
         v = s.read(not force)
-        GLib.idle_add(_update_setting_item, sb, v, online, sensitive, priority=99)
+        GLib.idle_add(_update_setting_item, sb, v, online, sensitive, True, priority=99)
 
     _ui_async(_do_read, setting, force_read, sbox, device_is_online, sensitive)
 
@@ -73,6 +72,21 @@ def _write_async(setting, value, sbox, sensitive=True, key=None):
 #
 
 
+class ComboBoxText(Gtk.ComboBoxText):
+
+    def get_value(self):
+        return int(self.get_active_id())
+
+    def set_value(self, value):
+        return self.set_active_id(str(int(value)))
+
+
+class Scale(Gtk.Scale):
+
+    def get_value(self):
+        return int(super().get_value())
+
+
 class Control():
 
     def __init__(**kwargs):
@@ -92,7 +106,8 @@ class Control():
     def layout(self, sbox, label, change, spinner, failed):
         sbox.pack_start(label, False, False, 0)
         sbox.pack_end(change, False, False, 0)
-        sbox.pack_end(self, sbox.setting.kind == _SETTING_KIND.range, sbox.setting.kind == _SETTING_KIND.range, 0)
+        fill = sbox.setting.kind == _SETTING_KIND.range or sbox.setting.kind == _SETTING_KIND.hetero
+        sbox.pack_end(self, fill, fill, 0)
         sbox.pack_end(spinner, False, False, 0)
         sbox.pack_end(failed, False, False, 0)
         return self
@@ -106,7 +121,8 @@ class ToggleControl(Gtk.Switch, Control):
         self.connect('notify::active', self.changed)
 
     def set_value(self, value):
-        self.set_state(value)
+        if value is not None:
+            self.set_state(value)
 
     def get_value(self):
         return self.get_state()
@@ -161,7 +177,8 @@ class ChoiceControlLittle(Gtk.ComboBoxText, Control):
         return int(self.get_active_id()) if self.get_active_id() is not None else None
 
     def set_value(self, value):
-        self.set_active_id(str(int(value)))
+        if value is not None:
+            self.set_active_id(str(int(value)))
 
     def get_choice(self):
         id = self.get_value()
@@ -199,7 +216,8 @@ class ChoiceControlBig(Gtk.Entry, Control):
         return int(choice) if choice is not None else None
 
     def set_value(self, value):
-        self.set_text(str(next((x for x in self.choices if x == value), None)))
+        if value is not None:
+            self.set_text(str(next((x for x in self.choices if x == value), None)))
 
     def get_choice(self):
         key = self.get_text()
@@ -246,6 +264,8 @@ class MapChoiceControl(Gtk.HBox, Control):
             return self.valueBox.get_value()
 
     def set_value(self, value):
+        if value is None:
+            return
         self.valueBox.set_sensitive(self.get_sensitive())
         key = int(self.keyBox.get_active_id())
         if value.get(key) is not None:
@@ -352,6 +372,8 @@ class MultipleToggleControl(MultipleControl):
                 _write_async(self.sbox.setting, new_state, self.sbox, key=int(key))
 
     def set_value(self, value):
+        if value is None:
+            return
         active = 0
         total = len(self._label_control_pairs)
         to_join = []
@@ -434,6 +456,8 @@ class MultipleRangeControl(MultipleControl):
             _write_async(self.sbox.setting, self.sbox.setting._value[int(item)], self.sbox, key=int(item))
 
     def set_value(self, value):
+        if value is None:
+            return
         b = ''
         n = 0
         for ch in self._items:
@@ -494,6 +518,8 @@ class PackedRangeControl(MultipleRangeControl):
             _write_async(self.sbox.setting, self.sbox.setting._value[int(item)], self.sbox, key=int(item))
 
     def set_value(self, value):
+        if value is None:
+            return
         b = ''
         n = len(self._items)
         for h in self._items:
@@ -509,6 +535,79 @@ class PackedRangeControl(MultipleRangeControl):
         self._button.set_tooltip_text(b)
 
 
+# control with an ID key that determines what else to show
+class HeteroKeyControl(Gtk.HBox, Control):
+
+    def __init__(self, sbox, delegate=None):
+        super().__init__(homogeneous=False, spacing=6)
+        self.init(sbox, delegate)
+        self._items = {}
+        for item in sbox.setting.possible_fields:
+            if item['label']:
+                item_lblbox = Gtk.Label(item['label'])
+                self.pack_start(item_lblbox, False, False, 0)
+                item_lblbox.set_visible(False)
+            else:
+                item_lblbox = None
+            if item['kind'] == _SETTING_KIND.choice:
+                item_box = ComboBoxText()
+                for entry in item['choices']:
+                    item_box.append(str(int(entry)), str(entry))
+                item_box.set_active(0)
+                item_box.connect('changed', self.changed)
+                self.pack_start(item_box, False, False, 0)
+            elif item['kind'] == _SETTING_KIND.range:
+                item_box = Scale()
+                item_box.set_range(item['min'], item['max'])
+                item_box.set_round_digits(0)
+                item_box.set_digits(0)
+                item_box.set_increments(1, 5)
+                item_box.connect('value-changed', self.changed)
+                self.pack_start(item_box, True, True, 0)
+            item_box.set_visible(False)
+            self._items[str(item['name'])] = (item_lblbox, item_box)
+
+    def get_value(self):
+        result = {}
+        for k, (_lblbox, box) in self._items.items():
+            result[str(k)] = box.get_value()
+        result = _LEDEffectSetting(**result)
+        return result
+
+    def set_value(self, value):
+        self.set_sensitive(False)
+        if value is not None:
+            for k, v in value.__dict__.items():
+                if k in self._items:
+                    (lblbox, box) = self._items[k]
+                    box.set_value(v)
+        self.setup_visibles(value.ID if value is not None else 0)
+
+    def setup_visibles(self, ID):
+        fields = self.sbox.setting.fields_map[ID][1] if ID in self.sbox.setting.fields_map else {}
+        for name, (lblbox, box) in self._items.items():
+            visible = name in fields or name == 'ID'
+            if lblbox:
+                lblbox.set_visible(visible)
+            box.set_visible(visible)
+
+    def changed(self, control):
+        if self.get_sensitive() and control.get_sensitive():
+            if 'ID' in self._items and control == self._items['ID'][1]:
+                self.setup_visibles(int(self._items['ID'][1].get_value()))
+            if hasattr(control, '_timer'):
+                control._timer.cancel()
+            control._timer = _Timer(0.3, lambda: GLib.idle_add(self._write, control))
+            control._timer.start()
+
+    def _write(self, control):
+        control._timer.cancel()
+        delattr(control, '_timer')
+        new_state = self.get_value()
+        if self.sbox.setting._value != new_state:
+            _write_async(self.sbox.setting, new_state, self.sbox)
+
+
 #
 #
 #
@@ -590,9 +689,11 @@ def _create_sbox(s, device):
         control = MultipleRangeControl(sbox, change)
     elif s.kind == _SETTING_KIND.packed_range:
         control = PackedRangeControl(sbox, change)
+    elif s.kind == _SETTING_KIND.hetero:
+        control = HeteroKeyControl(sbox, change)
     else:
-        if _log.isEnabledFor(_WARNING):
-            _log.warn('setting %s display not implemented', s.label)
+        if logger.isEnabledFor(logging.WARNING):
+            logger.warning('setting %s display not implemented', s.label)
         return None
 
     control.set_sensitive(False)  # the first read will enable it
@@ -604,18 +705,18 @@ def _create_sbox(s, device):
     return sbox
 
 
-def _update_setting_item(sbox, value, is_online=True, sensitive=True):
+def _update_setting_item(sbox, value, is_online=True, sensitive=True, nullOK=False):
     #    sbox._spinner.set_visible(False)   # don't repack item box
     sbox._spinner.stop()
-    if value is None:
-        sbox._control.set_sensitive(False)
-        _change_icon(False, sbox._change_icon)
+    sensitive = sbox._change_icon._allowed if sensitive is None else sensitive
+    if value is None and not nullOK:
+        sbox._control.set_sensitive(sensitive is True)
+        _change_icon(sensitive, sbox._change_icon)
         sbox._failed.set_visible(is_online)
         return
     sbox._failed.set_visible(False)
     sbox._control.set_sensitive(False)
     sbox._control.set_value(value)
-    sensitive = sbox._change_icon._allowed if sensitive is None else sensitive
     sbox._control.set_sensitive(sensitive is True)
     _change_icon(sensitive, sbox._change_icon)
 
@@ -639,13 +740,13 @@ _items = {}
 def create():
     global _box
     assert _box is None
-    _box = Gtk.VBox(homogeneous=False, spacing=8)
+    _box = Gtk.VBox(homogeneous=False, spacing=4)
     _box._last_device = None
 
     config_scroll = Gtk.ScrolledWindow()
     config_scroll.add(_box)
     config_scroll.set_policy(Gtk.PolicyType.NEVER, Gtk.PolicyType.AUTOMATIC)
-    config_scroll.set_shadow_type(Gtk.ShadowType.IN)
+    config_scroll.set_shadow_type(Gtk.ShadowType.NONE)  # was IN
     config_scroll.set_size_request(0, 350)  # ask for enough vertical space for about eight settings
 
     return config_scroll
@@ -718,20 +819,26 @@ def _change_setting(device, setting, values):
 
 
 def record_setting(device, setting, values):
-    """External interface to record a setting that has changed on the device and have the GUI show the change"""
-    assert device == setting._device
+    """External interface to have the GUI show a change to a setting. Doesn't write to the device"""
     GLib.idle_add(_record_setting, device, setting, values, priority=99)
 
 
-def _record_setting(device, setting, values):
-    if len(values) > 1:
-        setting.update_key_value(values[0], values[-1])
-        value = {values[0]: values[-1]}
-    else:
-        setting.update(values[-1])
-        value = values[-1]
-    device_path = device.receiver.path if device.receiver else device.path
-    if (device_path, device.number, setting.name) in _items:
-        sbox = _items[(device_path, device.number, setting.name)]
-        if sbox:
-            _update_setting_item(sbox, value)
+def _record_setting(device, setting_class, values):
+    if logger.isEnabledFor(logging.DEBUG):
+        logger.debug('on %s changing setting %s to %s', device, setting_class.name, values)
+    setting = next((s for s in device.settings if s.name == setting_class.name), None)
+    if setting is None and logger.isEnabledFor(logging.DEBUG):
+        logger.debug('No setting for %s found on %s when trying to record a change made elsewhere', setting_class.name, device)
+    if setting:
+        assert device == setting._device
+        if len(values) > 1:
+            setting.update_key_value(values[0], values[-1])
+            value = {values[0]: values[-1]}
+        else:
+            setting.update(values[-1])
+            value = values[-1]
+        device_path = device.receiver.path if device.receiver else device.path
+        if (device_path, device.number, setting.name) in _items:
+            sbox = _items[(device_path, device.number, setting.name)]
+            if sbox:
+                _update_setting_item(sbox, value, sensitive=None)
