@@ -16,15 +16,13 @@
 ## with this program; if not, write to the Free Software Foundation, Inc.,
 ## 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 
-from logging import DEBUG as _DEBUG
-from logging import getLogger
+import logging
 
 import solaar.gtk as gtk
 
 from gi.repository import Gtk
 
-_log = getLogger(__name__)
-del getLogger
+logger = logging.getLogger(__name__)
 
 #
 #
@@ -33,48 +31,11 @@ del getLogger
 _LARGE_SIZE = 64
 Gtk.IconSize.LARGE = Gtk.icon_size_register('large', _LARGE_SIZE, _LARGE_SIZE)
 # Gtk.IconSize.XLARGE = Gtk.icon_size_register('x-large', _LARGE_SIZE * 2, _LARGE_SIZE * 2)
-# print ("menu", int(Gtk.IconSize.MENU), Gtk.icon_size_lookup(Gtk.IconSize.MENU))
-# print ("small toolbar", int(Gtk.IconSize.SMALL_TOOLBAR), Gtk.icon_size_lookup(Gtk.IconSize.SMALL_TOOLBAR))
-# print ("button", int(Gtk.IconSize.BUTTON), Gtk.icon_size_lookup(Gtk.IconSize.BUTTON))
-# print ("large toolbar", int(Gtk.IconSize.LARGE_TOOLBAR), Gtk.icon_size_lookup(Gtk.IconSize.LARGE_TOOLBAR))
-# print ("dnd", int(Gtk.IconSize.DND), Gtk.icon_size_lookup(Gtk.IconSize.DND))
-# print ("dialog", int(Gtk.IconSize.DIALOG), Gtk.icon_size_lookup(Gtk.IconSize.DIALOG))
 
 TRAY_INIT = 'solaar-init'
 TRAY_OKAY = 'solaar'
 TRAY_ATTENTION = 'solaar-attention'
 
-
-def _look_for_application_icons():
-    import os.path as _path
-    import sys as _sys
-
-    from os import environ as _environ
-    if _log.isEnabledFor(_DEBUG):
-        _log.debug('sys.path[0] = %s', _sys.path[0])
-    prefix_share = _path.normpath(_path.join(_path.realpath(_sys.path[0]), '..'))
-    src_share = _path.normpath(_path.join(_path.realpath(_sys.path[0]), '..', 'share'))
-    local_share = _environ.get('XDG_DATA_HOME', _path.expanduser(_path.join('~', '.local', 'share')))
-    data_dirs = _environ.get('XDG_DATA_DIRS', '/usr/local/share:/usr/share')
-    repo_share = _path.normpath(_path.join(_path.dirname(__file__), '..', '..', '..', 'share'))
-    setuptools_share = _path.normpath(_path.join(_path.dirname(__file__), '..', '..', 'share'))
-    del _sys
-
-    share_solaar = [prefix_share] + list(
-        _path.join(x, 'solaar') for x in [src_share, local_share, setuptools_share, repo_share] + data_dirs.split(':')
-    )
-    for location in share_solaar:
-        location = _path.join(location, 'icons')
-        if _log.isEnabledFor(_DEBUG):
-            _log.debug('looking for icons in %s', location)
-
-        if _path.exists(_path.join(location, TRAY_ATTENTION + '.svg')):
-            yield location
-
-    del _environ
-    # del _path
-
-
 _default_theme = None
 
 
@@ -84,21 +45,18 @@ def _init_icon_paths():
         return
 
     _default_theme = Gtk.IconTheme.get_default()
-    for p in _look_for_application_icons():
-        _default_theme.prepend_search_path(p)
-        break  # only prepend one path - that's sufficient
-    if _log.isEnabledFor(_DEBUG):
-        _log.debug('icon theme paths: %s', _default_theme.get_search_path())
+    if logger.isEnabledFor(logging.DEBUG):
+        logger.debug('icon theme paths: %s', _default_theme.get_search_path())
 
     if gtk.battery_icons_style == 'symbolic':
         global TRAY_OKAY
         TRAY_OKAY = TRAY_INIT  # use monochrome tray icon
         if not _default_theme.has_icon('battery-good-symbolic'):
-            _log.warning('failed to detect symbolic icons')
+            logger.warning('failed to detect symbolic icons')
             gtk.battery_icons_style = 'regular'
     if gtk.battery_icons_style == 'regular':
         if not _default_theme.has_icon('battery-good'):
-            _log.warning('failed to detect icons')
+            logger.warning('failed to detect icons')
             gtk.battery_icons_style = 'solaar'
 
 
@@ -110,10 +68,10 @@ def _init_icon_paths():
 def battery(level=None, charging=False):
     icon_name = _battery_icon_name(level, charging)
     if not _default_theme.has_icon(icon_name):
-        _log.warning('icon %s not found in current theme', icon_name)
+        logger.warning('icon %s not found in current theme', icon_name)
         return TRAY_OKAY  # use Solaar icon if battery icon not available
-    elif _log.isEnabledFor(_DEBUG):
-        _log.debug('battery icon for %s:%s = %s', level, charging, icon_name)
+    elif logger.isEnabledFor(logging.DEBUG):
+        logger.debug('battery icon for %s:%s = %s', level, charging, icon_name)
     return icon_name
 
 
@@ -143,7 +101,7 @@ def _battery_icon_name(level, charging):
 def lux(level=None):
     if level is None or level < 0:
         return 'light_unknown'
-    return 'light_%03d' % (20 * ((level + 50) // 100))
+    return 'solaar-light_%03d' % (20 * ((level + 50) // 100))
 
 
 #
@@ -212,8 +170,8 @@ def icon_file(name, size=_LARGE_SIZE):
     theme_icon = _default_theme.lookup_icon(name, size, 0)
     if theme_icon:
         file_name = theme_icon.get_filename()
-        # if _log.isEnabledFor(_DEBUG):
-        #     _log.debug("icon %s(%d) => %s", name, size, file_name)
+        # if logger.isEnabledFor(logging.DEBUG):
+        #     logger.debug("icon %s(%d) => %s", name, size, file_name)
         return file_name
 
-    _log.warn('icon %s(%d) not found in current theme', name, size)
+    logger.warning('icon %s(%d) not found in current theme', name, size)
