@@ -16,10 +16,9 @@
 ## with this program; if not, write to the Free Software Foundation, Inc.,
 ## 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 
+import logging
 import os
 
-from logging import DEBUG as _DEBUG
-from logging import getLogger
 from time import time as _timestamp
 
 import solaar.gtk as gtk
@@ -35,8 +34,7 @@ from .about import show_window as _show_about_window
 from .window import popup as _window_popup
 from .window import toggle as _window_toggle
 
-_log = getLogger(__name__)
-del getLogger
+logger = logging.getLogger(__name__)
 
 #
 # constants
@@ -55,7 +53,7 @@ def _create_menu(quit_handler):
 
     # per-device menu entries will be generated as-needed
 
-    no_receiver = Gtk.MenuItem.new_with_label(_('No Logitech device found'))
+    no_receiver = Gtk.MenuItem.new_with_label(_('No supported device found'))
     no_receiver.set_sensitive(False)
     menu.append(no_receiver)
     menu.append(Gtk.SeparatorMenuItem.new())
@@ -84,9 +82,7 @@ def _scroll(tray_icon, event, direction=None):
         # ignore all other directions
         return
 
-    if len(_devices_info) < 4:
-        # don't bother with scrolling when there's only one receiver
-        # with only one or two devices
+    if sum(map(lambda i: i[1] is not None, _devices_info)) < 2:  # don't bother even trying to scroll if less than two devices
         return
 
     # scroll events come way too fast (at least 5-6 at once)
@@ -97,8 +93,8 @@ def _scroll(tray_icon, event, direction=None):
         return
     _last_scroll = now
 
-    # if _log.isEnabledFor(_DEBUG):
-    #     _log.debug("scroll direction %s", direction)
+    # if logger.isEnabledFor(logging.DEBUG):
+    #     logger.debug("scroll direction %s", direction)
 
     global _picked_device
     candidate = None
@@ -143,8 +139,8 @@ def _scroll(tray_icon, event, direction=None):
             _picked_device = None
 
     _picked_device = candidate or _picked_device
-    if _log.isEnabledFor(_DEBUG):
-        _log.debug('scroll: picked %s', _picked_device)
+    if logger.isEnabledFor(logging.DEBUG):
+        logger.debug('scroll: picked %s', _picked_device)
     _update_tray_icon()
 
 
@@ -152,22 +148,19 @@ try:
     import gi
     try:
         gi.require_version('AyatanaAppIndicator3', '0.1')
+        from gi.repository import AyatanaAppIndicator3 as AppIndicator3
         ayatana_appindicator_found = True
     except ValueError:
         try:
             gi.require_version('AppIndicator3', '0.1')
+            from gi.repository import AppIndicator3
             ayatana_appindicator_found = False
         except ValueError:
             # treat unavailable versions the same as unavailable packages
             raise ImportError
 
-    if ayatana_appindicator_found:
-        from gi.repository import AyatanaAppIndicator3 as AppIndicator3
-    else:
-        from gi.repository import AppIndicator3
-
-    if _log.isEnabledFor(_DEBUG):
-        _log.debug('using %sAppIndicator3' % ('Ayatana ' if ayatana_appindicator_found else ''))
+    if logger.isEnabledFor(logging.DEBUG):
+        logger.debug('using %sAppIndicator3' % ('Ayatana ' if ayatana_appindicator_found else ''))
 
     # Defense against AppIndicator3 bug that treats files in current directory as icon files
     # https://bugs.launchpad.net/ubuntu/+source/libappindicator/+bug/1363277
@@ -182,10 +175,8 @@ try:
 
     def _create(menu):
         _icons._init_icon_paths()
-        theme_paths = Gtk.IconTheme.get_default().get_search_path()
-
-        ind = AppIndicator3.Indicator.new_with_path(
-            'indicator-solaar', _icon_file(_icons.TRAY_INIT), AppIndicator3.IndicatorCategory.HARDWARE, theme_paths[0]
+        ind = AppIndicator3.Indicator.new(
+            'indicator-solaar', _icon_file(_icons.TRAY_INIT), AppIndicator3.IndicatorCategory.HARDWARE
         )
         ind.set_title(NAME)
         ind.set_status(AppIndicator3.IndicatorStatus.ACTIVE)
@@ -197,9 +188,12 @@ try:
 
         return ind
 
-    def _destroy(indicator):
+    def _hide(indicator):
         indicator.set_status(AppIndicator3.IndicatorStatus.PASSIVE)
 
+    def _show(indicator):
+        indicator.set_status(AppIndicator3.IndicatorStatus.ACTIVE)
+
     def _update_tray_icon():
         if _picked_device and gtk.battery_icons_style != 'solaar':
             _ignore, _ignore, name, device_status = _picked_device
@@ -232,8 +226,8 @@ try:
 
 except ImportError:
 
-    if _log.isEnabledFor(_DEBUG):
-        _log.debug('using StatusIcon')
+    if logger.isEnabledFor(logging.DEBUG):
+        logger.debug('using StatusIcon')
 
     def _create(menu):
         icon = Gtk.StatusIcon.new_from_icon_name(_icons.TRAY_INIT)
@@ -246,9 +240,12 @@ except ImportError:
 
         return icon
 
-    def _destroy(icon):
+    def _hide(icon):
         icon.set_visible(False)
 
+    def _show(icon):
+        icon.set_visible(True)
+
     def _update_tray_icon():
         tooltip_lines = _generate_tooltip_lines()
         tooltip = '\n'.join(tooltip_lines).rstrip('\n')
@@ -322,7 +319,6 @@ def _generate_description_lines():
                 yield '<b>%s</b> <small>(' % name + _('no status') + ')</small>'
             else:
                 yield '<b>%s</b> <small>(' % name + _('offline') + ')</small>'
-        yield ''
 
 
 def _pick_device_with_lowest_battery():
@@ -341,8 +337,8 @@ def _pick_device_with_lowest_battery():
             picked = info
             picked_level = level or 0
 
-    if _log.isEnabledFor(_DEBUG):
-        _log.debug('picked device with lowest battery: %s', picked)
+    if logger.isEnabledFor(logging.DEBUG):
+        logger.debug('picked device with lowest battery: %s', picked)
 
     return picked
 
@@ -405,7 +401,7 @@ def _add_receiver(receiver):
 
     new_menu_item = Gtk.ImageMenuItem.new_with_label(receiver.name)
     icon_set = _icons.device_icon_set(receiver.name)
-    new_menu_item.set_image(Gtk.Image().new_from_icon_set(icon_set, _MENU_ICON_SIZE))
+    new_menu_item.set_image(Gtk.Image().new_from_icon_name(icon_set.names[0], _MENU_ICON_SIZE))
     new_menu_item.show_all()
     new_menu_item.connect('activate', _window_popup, receiver.path)
     _menu.insert(new_menu_item, index)
@@ -426,7 +422,7 @@ def _remove_receiver(receiver):
 
 def _update_menu_item(index, device):
     if device is None or device.status is None:
-        _log.warn('updating an inactive device %s, assuming disconnected', device)
+        logger.warning('updating an inactive device %s, assuming disconnected', device)
         return None
 
     menu_items = _menu.get_children()
@@ -436,6 +432,7 @@ def _update_menu_item(index, device):
     charging = device.status.get(_K.BATTERY_CHARGING)
     icon_name = _icons.battery(level, charging)
 
+    menu_item.set_label(('  ' if 0 < device.number <= 6 else '') + device.name + ': ' + device.status.to_string())
     image_widget = menu_item.get_image()
     image_widget.set_sensitive(bool(device.online))
     _update_menu_icon(image_widget, icon_name)
@@ -463,13 +460,14 @@ def init(_quit_handler):
     _menu = _create_menu(_quit_handler)
     assert _icon is None
     _icon = _create(_menu)
+    update()
 
 
 def destroy():
     global _icon, _menu, _devices_info
     if _icon is not None:
         i, _icon = _icon, None
-        _destroy(i)
+        _hide(i)
         i = None
 
     _icon = None
@@ -525,3 +523,9 @@ def update(device=None):
         _picked_device = _pick_device_with_lowest_battery()
 
     _update_tray_icon()
+
+    if _icon:
+        if not _devices_info:
+            _hide(_icon)
+        else:
+            _show(_icon)
