@@ -16,8 +16,7 @@
 ## with this program; if not, write to the Free Software Foundation, Inc.,
 ## 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 
-from logging import DEBUG as _DEBUG
-from logging import getLogger
+import logging
 
 from gi.repository import Gdk, GLib, Gtk
 from gi.repository.GObject import TYPE_PYOBJECT
@@ -36,8 +35,7 @@ from . import icons as _icons
 from .about import show_window as _show_about_window
 from .diversion_rules import show_window as _show_diversion_window
 
-_log = getLogger(__name__)
-del getLogger
+logger = logging.getLogger(__name__)
 
 #
 # constants
@@ -68,25 +66,20 @@ assert len(_COLUMN_TYPES) == len(_COLUMN)
 
 
 def _new_button(label, icon_name=None, icon_size=_NORMAL_BUTTON_ICON_SIZE, tooltip=None, toggle=False, clicked=None):
-    if toggle:
-        b = Gtk.ToggleButton()
-    else:
-        b = Gtk.Button(label) if label else Gtk.Button()
-
+    b = Gtk.ToggleButton() if toggle else Gtk.Button()
+    c = Gtk.Box.new(Gtk.Orientation.HORIZONTAL, 5)
     if icon_name:
-        image = Gtk.Image.new_from_icon_name(icon_name, icon_size)
-        b.set_image(image)
-
+        c.pack_start(Gtk.Image.new_from_icon_name(icon_name, icon_size), True, True, 0)
+    if label:
+        c.pack_start(Gtk.Label(label), True, True, 0)
+    b.add(c)
+    if clicked is not None:
+        b.connect('clicked', clicked)
     if tooltip:
         b.set_tooltip_text(tooltip)
-
     if not label and icon_size < _NORMAL_BUTTON_ICON_SIZE:
         b.set_relief(Gtk.ReliefStyle.NONE)
         b.set_focus_on_click(False)
-
-    if clicked is not None:
-        b.connect('clicked', clicked)
-
     return b
 
 
@@ -392,8 +385,8 @@ def _find_selected_device_id():
 def _device_selected(selection):
     model, item = selection.get_selected()
     device = model.get_value(item, _COLUMN.DEVICE) if item else None
-    # if _log.isEnabledFor(_DEBUG):
-    #     _log.debug("window tree selected device %s", device)
+    # if logger.isEnabledFor(logging.DEBUG):
+    #     logger.debug("window tree selected device %s", device)
     if device:
         _update_info_panel(device, full=True)
     else:
@@ -423,8 +416,8 @@ def _receiver_row(receiver_path, receiver=None):
         status_icon = None
         row_data = (receiver_path, 0, True, receiver.name, icon_name, status_text, status_icon, receiver)
         assert len(row_data) == len(_TREE_SEPATATOR)
-        if _log.isEnabledFor(_DEBUG):
-            _log.debug('new receiver row %s', row_data)
+        if logger.isEnabledFor(logging.DEBUG):
+            logger.debug('new receiver row %s', row_data)
         item = _model.append(None, row_data)
         if _TREE_SEPATATOR:
             _model.append(None, _TREE_SEPATATOR)
@@ -448,7 +441,7 @@ def _device_row(receiver_path, device_number, device=None):
         new_child_index = 0
         while item:
             if _model.get_value(item, _COLUMN.PATH) != receiver_path:
-                _log.warn(
+                logger.warning(
                     'path for device row %s different from path for receiver %s', _model.get_value(item, _COLUMN.PATH),
                     receiver_path
                 )
@@ -469,8 +462,8 @@ def _device_row(receiver_path, device_number, device=None):
             receiver_path, device_number, bool(device.online), device.codename, icon_name, status_text, status_icon, device
         )
         assert len(row_data) == len(_TREE_SEPATATOR)
-        if _log.isEnabledFor(_DEBUG):
-            _log.debug('new device row %s at index %d', row_data, new_child_index)
+        if logger.isEnabledFor(logging.DEBUG):
+            logger.debug('new device row %s at index %d', row_data, new_child_index)
         item = _model.insert(receiver_row, new_child_index, row_data)
 
     return item or None
@@ -492,7 +485,7 @@ def select(receiver_path, device_number=None):
         selection = _tree.get_selection()
         selection.select_iter(item)
     else:
-        _log.warn('select(%s, %s) failed to find an item', receiver_path, device_number)
+        logger.warning('select(%s, %s) failed to find an item', receiver_path, device_number)
 
 
 def _hide(w, _ignore=None):
@@ -555,12 +548,7 @@ def _update_details(button):
                 hid_version = device.protocol
                 yield (_('Protocol'), 'HID++ %1.1f' % hid_version if hid_version else _('Unknown'))
                 if read_all and device.polling_rate:
-                    yield (
-                        _('Polling rate'), _('%(rate)d ms (%(rate_hz)dHz)') % {
-                            'rate': device.polling_rate,
-                            'rate_hz': 1000 // device.polling_rate
-                        }
-                    )
+                    yield (_('Polling rate'), device.polling_rate)
 
                 if read_all or not device.online:
                     yield (_('Serial'), device.serial)
@@ -585,11 +573,18 @@ def _update_details(button):
             _details._text.set_markup(text)
 
         def _make_text(items):
-            text = '\n'.join('%-13s: %s' % i for i in items)
+            text = '\n'.join('%-13s: %s' % (name, value) for name, value in items)
             return '<small><tt>' + text + '</tt></small>'
 
+        def _displayable_items(items):
+            for name, value in items:
+                value = GLib.markup_escape_text(str(value).replace('\x00', '')).strip()
+                if value:
+                    yield name, value
+
         def _read_slow(device):
             items = _details_items(selected_device, True)
+            items = _displayable_items(items)
             text = _make_text(items)
             if device == _details._current_device:
                 GLib.idle_add(_set_details, text)
@@ -616,7 +611,7 @@ def _update_receiver_panel(receiver, panel, buttons, full=False):
     devices_count = len(receiver)
 
     paired_text = _(
-        'No device paired.'
+        _('No device paired.')
     ) if devices_count == 0 else ngettext('%(count)s paired device.', '%(count)s paired devices.', devices_count) % {
         'count': devices_count
     }
@@ -630,7 +625,7 @@ def _update_receiver_panel(receiver, panel, buttons, full=False):
         }
     elif devices_count > 0:
         paired_text += '\n\n<small>%s</small>' % _('Only one device can be paired to this receiver.')
-    pairings = receiver.remaining_pairings(False)
+    pairings = receiver.remaining_pairings()
     if (pairings is not None and pairings >= 0):
         paired_text += '\n<small>%s</small>' % (
             ngettext('This receiver has %d pairing remaining.', 'This receiver has %d pairings remaining.', pairings) %
@@ -656,13 +651,9 @@ def _update_receiver_panel(receiver, panel, buttons, full=False):
     # b._insecure.set_visible(False)
     buttons._unpair.set_visible(False)
 
-    if (receiver.may_unpair or receiver.re_pairs) and not is_pairing and \
-       (receiver.remaining_pairings() is None or receiver.remaining_pairings() != 0):
-        if not receiver.re_pairs and devices_count >= receiver.max_devices:
-            paired_devices = tuple(n for n in range(1, receiver.max_devices + 1) if n in receiver)
-            buttons._pair.set_sensitive(len(paired_devices) < receiver.max_devices)
-        else:
-            buttons._pair.set_sensitive(True)
+    if not is_pairing and (receiver.remaining_pairings() is None or receiver.remaining_pairings() != 0) and \
+       (receiver.re_pairs or devices_count < receiver.max_devices):
+        buttons._pair.set_sensitive(True)
     else:
         buttons._pair.set_sensitive(False)
 
@@ -680,14 +671,9 @@ def _update_device_panel(device, panel, buttons, full=False):
     battery_level = device.status.get(_K.BATTERY_LEVEL)
     battery_voltage = device.status.get(_K.BATTERY_VOLTAGE)
     if battery_level is None and battery_voltage is None:
-        icon_name = _icons.battery()
-        panel._battery._icon.set_from_icon_name(icon_name, _INFO_ICON_SIZE)
-        panel._battery._icon.set_sensitive(False)
-        panel._battery._text.set_sensitive(is_online)
-        panel._battery._label.set_text(_('Battery'))
-        panel._battery._text.set_markup('<small>%s</small>' % _('unknown'))
-        panel._battery.set_tooltip_text(_('Battery information unknown.'))
+        panel._battery.set_visible(False)
     else:
+        panel._battery.set_visible(True)
         battery_next_level = device.status.get(_K.BATTERY_NEXT_LEVEL)
         charging = device.status.get(_K.BATTERY_CHARGING)
         icon_name = _icons.battery(battery_level, charging)
@@ -722,9 +708,16 @@ def _update_device_panel(device, panel, buttons, full=False):
         panel._battery._text.set_markup(text)
         panel._battery.set_tooltip_text(tooltip_text)
 
-    if is_online:
-        not_secure = device.status.get(_K.LINK_ENCRYPTED) is False
-        if not_secure:
+    if device.status.get(_K.LINK_ENCRYPTED) is None:
+        panel._secure.set_visible(False)
+    elif is_online:
+        panel._secure.set_visible(True)
+        panel._secure._icon.set_visible(True)
+        if device.status.get(_K.LINK_ENCRYPTED) is True:
+            panel._secure._text.set_text(_('encrypted'))
+            panel._secure._icon.set_from_icon_name('security-high', _INFO_ICON_SIZE)
+            panel._secure.set_tooltip_text(_('The wireless link between this device and its receiver is encrypted.'))
+        else:
             panel._secure._text.set_text(_('not encrypted'))
             panel._secure._icon.set_from_icon_name('security-low', _INFO_ICON_SIZE)
             panel._secure.set_tooltip_text(
@@ -733,14 +726,10 @@ def _update_device_panel(device, panel, buttons, full=False):
                     'This is a security issue for pointing devices, and a major security issue for text-input devices.'
                 )
             )
-        else:
-            panel._secure._text.set_text(_('encrypted'))
-            panel._secure._icon.set_from_icon_name('security-high', _INFO_ICON_SIZE)
-            panel._secure.set_tooltip_text(_('The wireless link between this device and its receiver is encrypted.'))
-        panel._secure._icon.set_visible(True)
     else:
-        panel._secure._text.set_markup('<small>%s</small>' % _('offline'))
+        panel._secure.set_visible(True)
         panel._secure._icon.set_visible(False)
+        panel._secure._text.set_markup('<small>%s</small>' % _('offline'))
         panel._secure.set_tooltip_text('')
 
     if is_online:
@@ -822,7 +811,6 @@ _window = None
 
 def init(show_window, hide_on_close):
     Gtk.Window.set_default_icon_name(NAME.lower())
-    Gtk.Window.set_default_icon_from_file(_icons.icon_file(NAME.lower()))
 
     global _model, _tree, _details, _info, _empty, _window
     _model = Gtk.TreeStore(*_COLUMN_TYPES)
@@ -919,6 +907,8 @@ def update_device(device, item, selected_device_id, need_popup, full=False):
         icon_name = _icons.battery(battery_level, charging)
         _model.set_value(item, _COLUMN.STATUS_ICON, icon_name)
 
+    _model.set_value(item, _COLUMN.NAME, device.codename)
+
     if selected_device_id is None or need_popup:
         select(device.receiver.path if device.receiver else device.path, device.number)
     elif selected_device_id == (device.receiver.path if device.receiver else device.path, device.number):
